@@ -1,60 +1,54 @@
-/**************************************************************************
+/****************************************************************************
 **
-** This file is part of Qt Creator
+** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
-**
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** This file is part of Qt Creator.
 **
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** This file may be used under the terms of the GNU Lesser General Public
-** License version 2.1 as published by the Free Software Foundation and
-** appearing in the file LICENSE.LGPL included in the packaging of this file.
-** Please review the following information to ensure the GNU Lesser General
-** Public License version 2.1 requirements will be met:
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Nokia gives you certain additional
-** rights. These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** Other Usage
-**
-** Alternatively, this file may be used in accordance with the terms and
-** conditions contained in a signed written agreement between you and Nokia.
-**
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
-**
-**************************************************************************/
-
-
-/// TODO
-/// To check
-/// a) with an old cmake
-/// => should not show combobox always use mingw generator
-/// b) with an new cmake
-/// always show combo box, defaulting if there's already a existing build
-
+****************************************************************************/
 
 #include "cmakeopenprojectwizard.h"
 #include "cmakeprojectmanager.h"
+#include "cmakebuildconfiguration.h"
 
+#include <coreplugin/icore.h>
 #include <utils/pathchooser.h>
-#include <projectexplorer/toolchainmanager.h>
+#include <utils/fancylineedit.h>
+#include <projectexplorer/kitinformation.h>
+#include <projectexplorer/kitmanager.h>
 #include <projectexplorer/toolchain.h>
 #include <projectexplorer/abi.h>
 #include <texteditor/fontsettings.h>
 
-#include <QtGui/QVBoxLayout>
-#include <QtGui/QFormLayout>
-#include <QtGui/QLabel>
-#include <QtGui/QPushButton>
-#include <QtGui/QPlainTextEdit>
-#include <QtCore/QDateTime>
-#include <QtCore/QStringList>
+#include <QVBoxLayout>
+#include <QFormLayout>
+#include <QLabel>
+#include <QPushButton>
+#include <QPlainTextEdit>
+#include <QDateTime>
+#include <QSettings>
+#include <QStringList>
+#include <QApplication>
 
 using namespace CMakeProjectManager;
 using namespace CMakeProjectManager::Internal;
@@ -70,72 +64,194 @@ using namespace CMakeProjectManager::Internal;
 //                                   |--> Already existing cbp file (and new enough) --> Page: Ready to load the project
 //                                   |--> Page: Ask for cmd options, run generator
 
-CMakeOpenProjectWizard::CMakeOpenProjectWizard(CMakeManager *cmakeManager, const QString &sourceDirectory, const Utils::Environment &env)
+
+namespace CMakeProjectManager {
+namespace Internal {
+    class GeneratorInfo
+    {
+    public:
+        enum Ninja { NoNinja, OfferNinja, ForceNinja };
+        static QList<GeneratorInfo> generatorInfosFor(ProjectExplorer::Kit *k, Ninja n, bool hasCodeBlocks);
+
+        GeneratorInfo();
+        explicit GeneratorInfo(ProjectExplorer::Kit *kit, bool ninja = false);
+
+        ProjectExplorer::Kit *kit() const;
+        bool isNinja() const;
+
+        QString displayName() const;
+        QString generatorArgument() const;
+        QString generator() const;
+
+    private:
+        ProjectExplorer::Kit *m_kit;
+        bool m_isNinja;
+    };
+
+}
+}
+
+Q_DECLARE_METATYPE(CMakeProjectManager::Internal::GeneratorInfo);
+
+GeneratorInfo::GeneratorInfo()
+    : m_kit(0), m_isNinja(false)
+{}
+
+GeneratorInfo::GeneratorInfo(ProjectExplorer::Kit *kit, bool ninja)
+    : m_kit(kit), m_isNinja(ninja)
+{}
+
+ProjectExplorer::Kit *GeneratorInfo::kit() const
+{
+    return m_kit;
+}
+
+bool GeneratorInfo::isNinja() const {
+    return m_isNinja;
+}
+
+QString GeneratorInfo::generator() const
+{
+    if (!m_kit)
+        return QString();
+    ProjectExplorer::ToolChain *tc = ProjectExplorer::ToolChainKitInformation::toolChain(m_kit);
+    ProjectExplorer::Abi targetAbi = tc->targetAbi();
+    if (m_isNinja) {
+        return QLatin1String("Ninja");
+    } else if (targetAbi.os() == ProjectExplorer::Abi::WindowsOS) {
+        if (targetAbi.osFlavor() == ProjectExplorer::Abi::WindowsMsvc2005Flavor
+                || targetAbi.osFlavor() == ProjectExplorer::Abi::WindowsMsvc2008Flavor
+                || targetAbi.osFlavor() == ProjectExplorer::Abi::WindowsMsvc2010Flavor
+                || targetAbi.osFlavor() == ProjectExplorer::Abi::WindowsMsvc2012Flavor) {
+            return QLatin1String("NMake Makefiles");
+        } else if (targetAbi.osFlavor() == ProjectExplorer::Abi::WindowsMSysFlavor) {
+#ifdef Q_OS_WIN
+                return QLatin1String("MinGW Makefiles");
+#else
+                return QLatin1String("Unix Makefiles");
+#endif
+        }
+    }
+    return QLatin1String("Unix Makefiles");
+}
+
+QString GeneratorInfo::generatorArgument() const
+{
+    QString tmp = generator();
+    if (tmp.isEmpty())
+        return tmp;
+    return QLatin1String("-GCodeBlocks - ") + tmp;
+}
+
+QString GeneratorInfo::displayName() const
+{
+    if (!m_kit)
+        return QString();
+    if (m_isNinja)
+        return QApplication::tr("Ninja (%1)").arg(m_kit->displayName());
+    ProjectExplorer::ToolChain *tc = ProjectExplorer::ToolChainKitInformation::toolChain(m_kit);
+    ProjectExplorer::Abi targetAbi = tc->targetAbi();
+    if (targetAbi.os() == ProjectExplorer::Abi::WindowsOS) {
+        if (targetAbi.osFlavor() == ProjectExplorer::Abi::WindowsMsvc2005Flavor
+                || targetAbi.osFlavor() == ProjectExplorer::Abi::WindowsMsvc2008Flavor
+                || targetAbi.osFlavor() == ProjectExplorer::Abi::WindowsMsvc2010Flavor
+                || targetAbi.osFlavor() == ProjectExplorer::Abi::WindowsMsvc2012Flavor) {
+            return QApplication::tr("NMake Generator (%1)").arg(m_kit->displayName());
+        } else if (targetAbi.osFlavor() == ProjectExplorer::Abi::WindowsMSysFlavor) {
+#ifdef Q_OS_WIN
+                return QApplication::tr("MinGW Generator (%1)").arg(m_kit->displayName());
+#else
+                return QApplication::tr("Unix Generator (%1)").arg(m_kit->displayName());
+#endif
+        }
+    } else {
+        // Non windows
+        return QApplication::tr("Unix Generator (%1)").arg(m_kit->displayName());
+    }
+    return QString();
+}
+
+QList<GeneratorInfo> GeneratorInfo::generatorInfosFor(ProjectExplorer::Kit *k, Ninja n, bool hasCodeBlocks)
+{
+    QList<GeneratorInfo> results;
+    ProjectExplorer::ToolChain *tc = ProjectExplorer::ToolChainKitInformation::toolChain(k);
+    if (!tc)
+        return results;
+    ProjectExplorer::Abi targetAbi = tc->targetAbi();
+    if (n != ForceNinja) {
+        if (targetAbi.os() == ProjectExplorer::Abi::WindowsOS) {
+            if (targetAbi.osFlavor() == ProjectExplorer::Abi::WindowsMsvc2005Flavor
+                    || targetAbi.osFlavor() == ProjectExplorer::Abi::WindowsMsvc2008Flavor
+                    || targetAbi.osFlavor() == ProjectExplorer::Abi::WindowsMsvc2010Flavor
+                    || targetAbi.osFlavor() == ProjectExplorer::Abi::WindowsMsvc2012Flavor) {
+                if (hasCodeBlocks)
+                    results << GeneratorInfo(k);
+            } else if (targetAbi.osFlavor() == ProjectExplorer::Abi::WindowsMSysFlavor) {
+                results << GeneratorInfo(k);
+            }
+        } else {
+            // Non windows
+            results << GeneratorInfo(k);
+        }
+    }
+    if (n != NoNinja)
+        results << GeneratorInfo(k, true);
+    return results;
+}
+
+//////////////
+/// CMakeOpenProjectWizard
+//////////////
+
+CMakeOpenProjectWizard::CMakeOpenProjectWizard(CMakeManager *cmakeManager, const QString &sourceDirectory, Utils::Environment env)
     : m_cmakeManager(cmakeManager),
       m_sourceDirectory(sourceDirectory),
-      m_creatingCbpFiles(false),
       m_environment(env),
-      m_toolChain(0)
+      m_useNinja(false),
+      m_kit(0)
 {
-    int startid;
     if (hasInSourceBuild()) {
-        startid = InSourcePageId;
         m_buildDirectory = m_sourceDirectory;
+        addPage(new InSourceBuildPage(this));
     } else {
-        startid = ShadowBuildPageId;
-        QDir dir(m_sourceDirectory);
-        dir.cdUp();
-        m_buildDirectory = dir.absolutePath() + "/qtcreator-build";
+        m_buildDirectory = m_sourceDirectory + QLatin1String("-build");
+        addPage(new ShadowBuildPage(this));
     }
 
-    setPage(InSourcePageId, new InSourceBuildPage(this));
-    setPage(ShadowBuildPageId, new ShadowBuildPage(this));
-    setPage(CMakeRunPageId, new CMakeRunPage(this));
+    if (!m_cmakeManager->isCMakeExecutableValid())
+        addPage(new ChooseCMakePage(this));
 
-    Utils::WizardProgress *wp = wizardProgress();
-    Utils::WizardProgressItem *inSourceItem = wp->item(InSourcePageId);
-    Utils::WizardProgressItem *shadowBuildItem = wp->item(ShadowBuildPageId);
-    Utils::WizardProgressItem *cmakeRunItem = wp->item(CMakeRunPageId);
-    inSourceItem->setNextItems(QList<Utils::WizardProgressItem *>() << cmakeRunItem);
-    shadowBuildItem->setNextItems(QList<Utils::WizardProgressItem *>() << cmakeRunItem);
+    addPage(new CMakeRunPage(this));
 
-    setStartId(startid);
     init();
 }
 
-CMakeOpenProjectWizard::CMakeOpenProjectWizard(CMakeManager *cmakeManager, const QString &sourceDirectory,
-                                               const QString &buildDirectory, CMakeOpenProjectWizard::Mode mode,
-                                               const Utils::Environment &env)
+CMakeOpenProjectWizard::CMakeOpenProjectWizard(CMakeManager *cmakeManager, CMakeOpenProjectWizard::Mode mode,
+                                               const BuildInfo &info)
     : m_cmakeManager(cmakeManager),
-      m_sourceDirectory(sourceDirectory),
-      m_creatingCbpFiles(true),
-      m_environment(env),
-      m_toolChain(0)
+      m_sourceDirectory(info.sourceDirectory),
+      m_environment(info.environment),
+      m_useNinja(info.useNinja),
+      m_kit(info.kit)
 {
-
     CMakeRunPage::Mode rmode;
     if (mode == CMakeOpenProjectWizard::NeedToCreate)
         rmode = CMakeRunPage::Recreate;
     else if (mode == CMakeOpenProjectWizard::WantToUpdate)
         rmode = CMakeRunPage::WantToUpdate;
-    else
+    else if (mode == CMakeOpenProjectWizard::NeedToUpdate)
         rmode = CMakeRunPage::NeedToUpdate;
-    addPage(new CMakeRunPage(this, rmode, buildDirectory));
-    init();
-}
+    else
+        rmode = CMakeRunPage::ChangeDirectory;
 
-CMakeOpenProjectWizard::CMakeOpenProjectWizard(CMakeManager *cmakeManager, const QString &sourceDirectory,
-                                               const QString &oldBuildDirectory,
-                                               const Utils::Environment &env)
-    : m_cmakeManager(cmakeManager),
-      m_sourceDirectory(sourceDirectory),
-      m_creatingCbpFiles(true),
-      m_environment(env),
-      m_toolChain(0)
-{
-    m_buildDirectory = oldBuildDirectory;
-    addPage(new ShadowBuildPage(this, true));
-    addPage(new CMakeRunPage(this, CMakeRunPage::ChangeDirectory));
+    if (mode == CMakeOpenProjectWizard::ChangeDirectory) {
+        m_buildDirectory = info.buildDirectory;
+        addPage(new ShadowBuildPage(this, true));
+    }
+    if (!m_cmakeManager->isCMakeExecutableValid())
+        addPage(new ChooseCMakePage(this));
+
+    addPage(new CMakeRunPage(this, rmode, info.buildDirectory));
     init();
 }
 
@@ -143,7 +259,6 @@ void CMakeOpenProjectWizard::init()
 {
     setOption(QWizard::NoBackButtonOnStartPage);
     setWindowTitle(tr("CMake Wizard"));
-    setMinimumSize(800, 600);
 }
 
 CMakeManager *CMakeOpenProjectWizard::cmakeManager() const
@@ -151,20 +266,6 @@ CMakeManager *CMakeOpenProjectWizard::cmakeManager() const
     return m_cmakeManager;
 }
 
-int CMakeOpenProjectWizard::nextId() const
-{
-    if (m_creatingCbpFiles)
-        return QWizard::nextId();
-    int cid = currentId();
-    if (cid == InSourcePageId) {
-        return CMakeRunPageId;
-    } else if (cid == ShadowBuildPageId) {
-        return CMakeRunPageId;
-    }
-    return -1;
-}
-
-
 bool CMakeOpenProjectWizard::hasInSourceBuild() const
 {
     QFileInfo fi(m_sourceDirectory + "/CMakeCache.txt");
@@ -202,6 +303,16 @@ void CMakeOpenProjectWizard::setBuildDirectory(const QString &directory)
     m_buildDirectory = directory;
 }
 
+bool CMakeOpenProjectWizard::useNinja() const
+{
+    return m_useNinja;
+}
+
+void CMakeOpenProjectWizard::setUseNinja(bool b)
+{
+    m_useNinja = b;
+}
+
 QString CMakeOpenProjectWizard::arguments() const
 {
     return m_arguments;
@@ -212,22 +323,20 @@ void CMakeOpenProjectWizard::setArguments(const QString &args)
     m_arguments = args;
 }
 
-ProjectExplorer::ToolChain *CMakeOpenProjectWizard::toolChain() const
-{
-    return m_toolChain;
-}
-
-void CMakeOpenProjectWizard::setToolChain(ProjectExplorer::ToolChain *tc)
-{
-    m_toolChain = tc;
-}
-
-
 Utils::Environment CMakeOpenProjectWizard::environment() const
 {
     return m_environment;
 }
 
+ProjectExplorer::Kit *CMakeOpenProjectWizard::kit() const
+{
+    return m_kit;
+}
+
+void CMakeOpenProjectWizard::setKit(ProjectExplorer::Kit *kit)
+{
+    m_kit = kit;
+}
 
 InSourceBuildPage::InSourceBuildPage(CMakeOpenProjectWizard *cmakeWizard)
     : QWizardPage(cmakeWizard), m_cmakeWizard(cmakeWizard)
@@ -262,6 +371,7 @@ ShadowBuildPage::ShadowBuildPage(CMakeOpenProjectWizard *cmakeWizard, bool chang
     m_pc = new Utils::PathChooser(this);
     m_pc->setBaseDirectory(m_cmakeWizard->sourceDirectory());
     m_pc->setPath(m_cmakeWizard->buildDirectory());
+    m_pc->setExpectedKind(Utils::PathChooser::Directory);
     connect(m_pc, SIGNAL(changed(QString)), this, SLOT(buildDirectoryChanged()));
     fl->addRow(tr("Build directory:"), m_pc);
     setTitle(tr("Build Location"));
@@ -272,10 +382,66 @@ void ShadowBuildPage::buildDirectoryChanged()
     m_cmakeWizard->setBuildDirectory(m_pc->path());
 }
 
+ChooseCMakePage::ChooseCMakePage(CMakeOpenProjectWizard *cmakeWizard)
+    : QWizardPage(cmakeWizard), m_cmakeWizard(cmakeWizard)
+{
+    QFormLayout *fl = new QFormLayout;
+    fl->setFieldGrowthPolicy(QFormLayout::ExpandingFieldsGrow);
+    setLayout(fl);
+
+    m_cmakeLabel = new QLabel;
+    m_cmakeLabel->setWordWrap(true);
+    fl->addRow(m_cmakeLabel);
+    // Show a field for the user to enter
+    m_cmakeExecutable = new Utils::PathChooser(this);
+    m_cmakeExecutable->setExpectedKind(Utils::PathChooser::ExistingCommand);
+    fl->addRow(tr("cmake Executable:"), m_cmakeExecutable);
+
+    connect(m_cmakeExecutable, SIGNAL(editingFinished()),
+            this, SLOT(cmakeExecutableChanged()));
+    connect(m_cmakeExecutable, SIGNAL(browsingFinished()),
+            this, SLOT(cmakeExecutableChanged()));
+
+    setTitle(tr("Choose Cmake Executable"));
+}
+
+void ChooseCMakePage::updateErrorText()
+{
+    QString cmakeExecutable = m_cmakeWizard->cmakeManager()->cmakeExecutable();
+    if (m_cmakeWizard->cmakeManager()->isCMakeExecutableValid()) {
+        m_cmakeLabel->setText(tr("The cmake executable is valid."));
+    } else {
+        QString text = tr("Please specify the path to the cmake executable. No cmake executable was found in the path.");
+        if (!cmakeExecutable.isEmpty()) {
+            QFileInfo fi(cmakeExecutable);
+            if (!fi.exists())
+                text += tr(" The cmake executable (%1) does not exist.").arg(cmakeExecutable);
+            else if (!fi.isExecutable())
+                text += tr(" The path %1 is not a executable.").arg(cmakeExecutable);
+            else
+                text += tr(" The path %1 is not a valid cmake.").arg(cmakeExecutable);
+        }
+        m_cmakeLabel->setText(text);
+    }
+}
+
+void ChooseCMakePage::cmakeExecutableChanged()
+{
+    m_cmakeWizard->cmakeManager()->setCMakeExecutable(m_cmakeExecutable->path());
+    updateErrorText();
+    emit completeChanged();
+}
+
+bool ChooseCMakePage::isComplete() const
+{
+    return m_cmakeWizard->cmakeManager()->isCMakeExecutableValid();
+}
+
 CMakeRunPage::CMakeRunPage(CMakeOpenProjectWizard *cmakeWizard, Mode mode, const QString &buildDirectory)
     : QWizardPage(cmakeWizard),
       m_cmakeWizard(cmakeWizard),
       m_complete(false),
+      m_optionalCMake(false),
       m_mode(mode),
       m_buildDirectory(buildDirectory)
 {
@@ -293,32 +459,10 @@ void CMakeRunPage::initWidgets()
 
     fl->addRow(m_descriptionLabel);
 
-    if (m_cmakeWizard->cmakeManager()->isCMakeExecutableValid()) {
-        m_cmakeExecutable = 0;
-    } else {
-        QString text = tr("Please specify the path to the cmake executable. No cmake executable was found in the path.");
-        QString cmakeExecutable = m_cmakeWizard->cmakeManager()->cmakeExecutable();
-        if (!cmakeExecutable.isEmpty()) {
-            QFileInfo fi(cmakeExecutable);
-            if (!fi.exists())
-                text += tr(" The cmake executable (%1) does not exist.").arg(cmakeExecutable);
-            else if (!fi.isExecutable())
-                text += tr(" The path %1 is not a executable.").arg(cmakeExecutable);
-            else
-                text += tr(" The path %1 is not a valid cmake.").arg(cmakeExecutable);
-        }
-
-        QLabel *cmakeLabel = new QLabel(text);
-        cmakeLabel->setWordWrap(true);
-        fl->addRow(cmakeLabel);
-        // Show a field for the user to enter
-        m_cmakeExecutable = new Utils::PathChooser(this);
-        m_cmakeExecutable->setExpectedKind(Utils::PathChooser::ExistingCommand);
-        fl->addRow("cmake Executable:", m_cmakeExecutable);
-    }
-
     // Run CMake Line (with arguments)
-    m_argumentsLineEdit = new QLineEdit(this);
+    m_argumentsLineEdit = new Utils::FancyLineEdit(this);
+    m_argumentsLineEdit->setHistoryCompleter(QLatin1String("CMakeArgumentsLineEdit"));
+
     connect(m_argumentsLineEdit,SIGNAL(returnPressed()), this, SLOT(runCMake()));
     fl->addRow(tr("Arguments:"), m_argumentsLineEdit);
 
@@ -353,19 +497,44 @@ void CMakeRunPage::initWidgets()
     fl->addRow(m_exitCodeLabel);
 
     setTitle(tr("Run CMake"));
+    setMinimumSize(600, 400);
+}
+
+QString CMakeRunPage::cachedGeneratorFromFile(const QString &cache)
+{
+    QFile fi(cache);
+    if (fi.exists()) {
+        // Cache exists, then read it...
+        if (fi.open(QIODevice::ReadOnly | QIODevice::Text)) {
+            while (!fi.atEnd()) {
+                QString line = fi.readLine();
+                if (line.startsWith("CMAKE_GENERATOR:INTERNAL=")) {
+                    int splitpos = line.indexOf('=');
+                    if (splitpos != -1) {
+                        QString cachedGenerator = line.mid(splitpos + 1).trimmed();
+                        if (!cachedGenerator.isEmpty())
+                            return cachedGenerator;
+                    }
+                }
+            }
+        }
+    }
+    return QString();
 }
 
 void CMakeRunPage::initializePage()
 {
     if (m_mode == Initial) {
-        m_complete = m_cmakeWizard->existsUpToDateXmlFile();
+        bool upToDateXmlFile = m_cmakeWizard->existsUpToDateXmlFile();;
         m_buildDirectory = m_cmakeWizard->buildDirectory();
 
-        if (m_cmakeWizard->existsUpToDateXmlFile()) {
+        if (upToDateXmlFile) {
             m_descriptionLabel->setText(
                     tr("The directory %1 already contains a cbp file, which is recent enough. "
-                       "You can pass special arguments or change the used tool chain here and rerun CMake. "
+                       "You can pass special arguments and rerun CMake. "
                        "Or simply finish the wizard directly.").arg(m_buildDirectory));
+            m_optionalCMake = true;
+            m_complete = true;
         } else {
             m_descriptionLabel->setText(
                     tr("The directory %1 does not contain a cbp file. Qt Creator needs to create this file by running CMake. "
@@ -377,14 +546,14 @@ void CMakeRunPage::initializePage()
                                        "If you want to add additional command line arguments, "
                                        "add them below. Note that CMake remembers command "
                                        "line arguments from the previous runs.").arg(m_buildDirectory));
-    } else if(m_mode == CMakeRunPage::Recreate) {
+    } else if (m_mode == CMakeRunPage::Recreate) {
         m_descriptionLabel->setText(tr("The directory %1 specified in a build-configuration, "
                                        "does not contain a cbp file. Qt Creator needs to "
                                        "recreate this file, by running CMake. "
                                        "Some projects require command line arguments to "
                                        "the initial CMake call. Note that CMake remembers command "
                                        "line arguments from the previous runs.").arg(m_buildDirectory));
-    } else if(m_mode == CMakeRunPage::ChangeDirectory) {
+    } else if (m_mode == CMakeRunPage::ChangeDirectory) {
         m_buildDirectory = m_cmakeWizard->buildDirectory();
         m_descriptionLabel->setText(tr("Qt Creator needs to run CMake in the new build directory. "
                                        "Some projects require command line arguments to the "
@@ -393,110 +562,98 @@ void CMakeRunPage::initializePage()
         m_descriptionLabel->setText(tr("Refreshing cbp file in %1.").arg(m_buildDirectory));
     }
 
-
-    // Try figuring out generator and toolchain from CMakeCache.txt
-    QString cachedGenerator;
-    QString cmakeCxxCompiler;
-    QFile fi(m_buildDirectory + "/CMakeCache.txt");
-    if (fi.exists()) {
-        // Cache exists, then read it...
-        if (fi.open(QIODevice::ReadOnly | QIODevice::Text)) {
-            while (!fi.atEnd()) {
-                QString line = fi.readLine();
-                if (line.startsWith("CMAKE_GENERATOR:INTERNAL=")) {
-                    int splitpos = line.indexOf('=');
-                    if (splitpos != -1)
-                        cachedGenerator = line.mid(splitpos + 1).trimmed();
-                }
-                if (line.startsWith("CMAKE_CXX_COMPILER:FILEPATH=")) {
-                    int splitpos = line.indexOf("=");
-                    if (splitpos != -1)
-                        cmakeCxxCompiler = line.mid(splitpos +1).trimmed();
-                }
-                if (!cachedGenerator.isEmpty() && !cmakeCxxCompiler.isEmpty())
-                    break;
-            }
-        }
-    }
-
     // Build the list of generators/toolchains we want to offer
-    // todo restrict toolchains based on CMAKE_CXX_COMPILER ?
-    Q_UNUSED(cmakeCxxCompiler);
     m_generatorComboBox->clear();
-    bool hasCodeBlocksGenerator = m_cmakeWizard->cmakeManager()->hasCodeBlocksMsvcGenerator();
-    ProjectExplorer::Abi abi = ProjectExplorer::Abi::hostAbi();
-    abi = ProjectExplorer::Abi(abi.architecture(), abi.os(), ProjectExplorer::Abi::UnknownFlavor,
-                               abi.binaryFormat(), 0);
-    QList<ProjectExplorer::ToolChain *> tcs =
-            ProjectExplorer::ToolChainManager::instance()->findToolChains(abi);
 
-    foreach (ProjectExplorer::ToolChain *tc, tcs) {
-        ProjectExplorer::Abi targetAbi = tc->targetAbi();
-        QVariant tcVariant = qVariantFromValue(static_cast<void *>(tc));
-        if (targetAbi.os() == ProjectExplorer::Abi::WindowsOS) {
-            if (targetAbi.osFlavor() == ProjectExplorer::Abi::WindowsMsvc2005Flavor
-                    || targetAbi.osFlavor() == ProjectExplorer::Abi::WindowsMsvc2008Flavor
-                    || targetAbi.osFlavor() == ProjectExplorer::Abi::WindowsMsvc2010Flavor) {
-                if (hasCodeBlocksGenerator && (cachedGenerator.isEmpty() || cachedGenerator == "NMake Makefiles"))
-                    m_generatorComboBox->addItem(tr("NMake Generator (%1)").arg(tc->displayName()), tcVariant);
-             } else if (targetAbi.osFlavor() == ProjectExplorer::Abi::WindowsMSysFlavor) {
-                if (cachedGenerator.isEmpty() || cachedGenerator == "MinGW Makefiles")
-                    m_generatorComboBox->addItem(tr("MinGW Generator (%1)").arg(tc->displayName()), tcVariant);
-            }
-        } else {
-            // Non windows
-            if (cachedGenerator.isEmpty() || cachedGenerator == "Unix Makefiles")
-                m_generatorComboBox->addItem(tr("Unix Generator (%1)").arg(tc->displayName()), tcVariant);
+    bool hasCodeBlocksGenerator = m_cmakeWizard->cmakeManager()->hasCodeBlocksMsvcGenerator();
+    bool hasNinjaGenerator = m_cmakeWizard->cmakeManager()->hasCodeBlocksNinjaGenerator();
+
+    if (m_mode == Initial) {
+        // Try figuring out generator and toolchain from CMakeCache.txt
+        QString cachedGenerator = cachedGeneratorFromFile(m_buildDirectory + "/CMakeCache.txt");
+
+        m_generatorComboBox->show();
+        QList<ProjectExplorer::Kit *> kitList =
+                ProjectExplorer::KitManager::instance()->kits();
+
+        foreach (ProjectExplorer::Kit *k, kitList) {
+            QList<GeneratorInfo> infos = GeneratorInfo::generatorInfosFor(k,
+                                                                          hasNinjaGenerator ? GeneratorInfo::OfferNinja : GeneratorInfo::NoNinja,
+                                                                          hasCodeBlocksGenerator);
+
+            foreach (const GeneratorInfo &info, infos)
+                if (cachedGenerator.isEmpty() || info.generator() == cachedGenerator)
+                    m_generatorComboBox->addItem(info.displayName(), qVariantFromValue(info));
         }
+    } else {
+        // Note: We don't compare the actually cached generator to what is set in the buildconfiguration
+        // We assume that the buildconfiguration is correct
+        GeneratorInfo::Ninja ninja;
+        if (m_mode == CMakeRunPage::NeedToUpdate || m_mode == CMakeRunPage::WantToUpdate) {
+            ninja = m_cmakeWizard->useNinja() ? GeneratorInfo::ForceNinja : GeneratorInfo::NoNinja;
+        } else { // Recreate, ChangeDirectory
+            // Note: ReCreate is technically just a removed .cbp file, we assume the cache
+            // got removed too. If the cache still exists the error message from cmake should
+            // be a good hint to change the generator
+            ninja = hasNinjaGenerator ? GeneratorInfo::OfferNinja : GeneratorInfo::NoNinja;
+        }
+
+        QList<GeneratorInfo> infos = GeneratorInfo::generatorInfosFor(m_cmakeWizard->kit(),
+                                                                      ninja,
+                                                                      true);
+        foreach (const GeneratorInfo &info, infos)
+                m_generatorComboBox->addItem(info.displayName(), qVariantFromValue(info));
     }
 }
 
+bool CMakeRunPage::validatePage()
+{
+    if (m_optionalCMake) {
+        int index = m_generatorComboBox->currentIndex();
+        if (index == -1)
+            return false;
+        GeneratorInfo generatorInfo = m_generatorComboBox->itemData(index).value<GeneratorInfo>();
+        m_cmakeWizard->setKit(generatorInfo.kit());
+        m_cmakeWizard->setUseNinja(generatorInfo.isNinja());
+    }
+    return QWizardPage::validatePage();
+}
+
 void CMakeRunPage::runCMake()
 {
-    if (m_cmakeExecutable) {
-        // We asked the user for the cmake executable
-        m_cmakeWizard->cmakeManager()->setCMakeExecutable(m_cmakeExecutable->path());
-    }
+    m_optionalCMake = false;
+    m_complete = false;
 
+    Utils::Environment env = m_cmakeWizard->environment();
     int index = m_generatorComboBox->currentIndex();
 
-    ProjectExplorer::ToolChain *tc = 0;
-    if (index >= 0)
-        tc = static_cast<ProjectExplorer::ToolChain *>(m_generatorComboBox->itemData(index).value<void *>());
-    if (!tc) {
+    if (index == -1) {
         m_output->appendPlainText(tr("No generator selected."));
         return;
     }
+    GeneratorInfo generatorInfo = m_generatorComboBox->itemData(index).value<GeneratorInfo>();
+    m_cmakeWizard->setKit(generatorInfo.kit());
+    m_cmakeWizard->setUseNinja(generatorInfo.isNinja());
 
-    m_cmakeWizard->setToolChain(tc);
+    // If mode is initial the user chooses the kit, otherwise it's already choosen
+    // and the environment already contains the kit
+    if (m_mode == Initial)
+        generatorInfo.kit()->addToEnvironment(env);
 
     m_runCMake->setEnabled(false);
     m_argumentsLineEdit->setEnabled(false);
     m_generatorComboBox->setEnabled(false);
-    CMakeManager *cmakeManager = m_cmakeWizard->cmakeManager();
-
-    QString generator = QLatin1String("-GCodeBlocks - Unix Makefiles");
-    if (tc->targetAbi().os() == ProjectExplorer::Abi::WindowsOS) {
-        if (tc->targetAbi().osFlavor() == ProjectExplorer::Abi::WindowsMSysFlavor)
-            generator = QLatin1String("-GCodeBlocks - MinGW Makefiles");
-        else
-            generator = QLatin1String("-GCodeBlocks - NMake Makefiles");
-    }
-
-
-    Utils::Environment env = m_cmakeWizard->environment();
-    tc->addToEnvironment(env);
-
-
 
     m_output->clear();
 
+    CMakeManager *cmakeManager = m_cmakeWizard->cmakeManager();
     if (m_cmakeWizard->cmakeManager()->isCMakeExecutableValid()) {
         m_cmakeProcess = new Utils::QtcProcess();
         connect(m_cmakeProcess, SIGNAL(readyReadStandardOutput()), this, SLOT(cmakeReadyReadStandardOutput()));
         connect(m_cmakeProcess, SIGNAL(readyReadStandardError()), this, SLOT(cmakeReadyReadStandardError()));
         connect(m_cmakeProcess, SIGNAL(finished(int)), this, SLOT(cmakeFinished()));
-        cmakeManager->createXmlFile(m_cmakeProcess, m_argumentsLineEdit->text(), m_cmakeWizard->sourceDirectory(), m_buildDirectory, env, generator);
+        cmakeManager->createXmlFile(m_cmakeProcess, m_argumentsLineEdit->text(), m_cmakeWizard->sourceDirectory(),
+                                    m_buildDirectory, env, generatorInfo.generatorArgument());
     } else {
         m_runCMake->setEnabled(true);
         m_argumentsLineEdit->setEnabled(true);
@@ -555,7 +712,6 @@ void CMakeRunPage::cmakeFinished()
     m_cmakeProcess->deleteLater();
     m_cmakeProcess = 0;
     m_cmakeWizard->setArguments(m_argumentsLineEdit->text());
-    //TODO Actually test that running cmake was finished, for setting this bool
     emit completeChanged();
 }
 
