@@ -1,49 +1,52 @@
-/**************************************************************************
+/****************************************************************************
 **
-** This file is part of Qt Creator
+** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
-**
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** This file is part of Qt Creator.
 **
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** This file may be used under the terms of the GNU Lesser General Public
-** License version 2.1 as published by the Free Software Foundation and
-** appearing in the file LICENSE.LGPL included in the packaging of this file.
-** Please review the following information to ensure the GNU Lesser General
-** Public License version 2.1 requirements will be met:
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Nokia gives you certain additional
-** rights. These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** Other Usage
-**
-** Alternatively, this file may be used in accordance with the terms and
-** conditions contained in a signed written agreement between you and Nokia.
-**
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
-**
-**************************************************************************/
+****************************************************************************/
 
 #include "searchresultwidget.h"
 #include "searchresulttreeview.h"
 #include "searchresulttreemodel.h"
 #include "searchresulttreeitems.h"
+#include "searchresulttreeitemroles.h"
 
 #include "ifindsupport.h"
+#include "treeviewfind.h"
 
 #include <aggregation/aggregate.h>
 #include <coreplugin/icore.h>
 
-#include <QtCore/QDir>
-#include <QtCore/QSettings>
-#include <QtGui/QVBoxLayout>
-#include <QtGui/QHBoxLayout>
+#include <QDir>
+#include <QSettings>
+#include <QVBoxLayout>
+#include <QHBoxLayout>
+
+static const int SEARCHRESULT_WARNING_LIMIT = 200000;
+static const char UNDO_WARNING_ID[] = "warninglabel";
+static const char SIZE_WARNING_ID[] = "sizeWarningLabel";
 
 namespace Find {
 namespace Internal {
@@ -66,139 +69,6 @@ public slots:
     void updateGeometry() { QLineEdit::updateGeometry(); }
 };
 
-class SearchResultFindSupport : public IFindSupport
-{
-    Q_OBJECT
-public:
-    SearchResultFindSupport(SearchResultTreeView *view)
-        : m_view(view),
-          m_incrementalWrappedState(false)
-    {
-    }
-
-    bool supportsReplace() const { return false; }
-
-    Find::FindFlags supportedFindFlags() const
-    {
-        return Find::FindBackward | Find::FindCaseSensitively
-                | Find::FindRegularExpression | Find::FindWholeWords;
-    }
-
-    void resetIncrementalSearch()
-    {
-        m_incrementalFindStart = QModelIndex();
-        m_incrementalWrappedState = false;
-    }
-
-    void clearResults() { }
-
-    QString currentFindString() const
-    {
-        return QString();
-    }
-
-    QString completedFindString() const
-    {
-        return QString();
-    }
-
-    void highlightAll(const QString &txt, Find::FindFlags findFlags)
-    {
-        Q_UNUSED(txt)
-        Q_UNUSED(findFlags)
-        return;
-    }
-
-    IFindSupport::Result findIncremental(const QString &txt, Find::FindFlags findFlags)
-    {
-        if (!m_incrementalFindStart.isValid()) {
-            m_incrementalFindStart = m_view->currentIndex();
-            m_incrementalWrappedState = false;
-        }
-        m_view->setCurrentIndex(m_incrementalFindStart);
-        bool wrapped = false;
-        IFindSupport::Result result = find(txt, findFlags, &wrapped);
-        if (wrapped != m_incrementalWrappedState) {
-            m_incrementalWrappedState = wrapped;
-            showWrapIndicator(m_view);
-        }
-        return result;
-    }
-
-    IFindSupport::Result findStep(const QString &txt, Find::FindFlags findFlags)
-    {
-        bool wrapped = false;
-        IFindSupport::Result result = find(txt, findFlags, &wrapped);
-        if (wrapped)
-            showWrapIndicator(m_view);
-        if (result == IFindSupport::Found) {
-            m_incrementalFindStart = m_view->currentIndex();
-            m_incrementalWrappedState = false;
-        }
-        return result;
-    }
-
-    IFindSupport::Result find(const QString &txt, Find::FindFlags findFlags, bool *wrapped)
-    {
-        if (wrapped)
-            *wrapped = false;
-        if (txt.isEmpty())
-            return IFindSupport::NotFound;
-        QModelIndex index;
-        if (findFlags & Find::FindRegularExpression) {
-            bool sensitive = (findFlags & Find::FindCaseSensitively);
-            index = m_view->model()->find(QRegExp(txt, (sensitive ? Qt::CaseSensitive : Qt::CaseInsensitive)),
-                                          m_view->currentIndex(),
-                                          Find::textDocumentFlagsForFindFlags(findFlags),
-                                          wrapped);
-        } else {
-            index = m_view->model()->find(txt,
-                                          m_view->currentIndex(),
-                                          Find::textDocumentFlagsForFindFlags(findFlags),
-                                          wrapped);
-        }
-        if (index.isValid()) {
-            m_view->setCurrentIndex(index);
-            m_view->scrollTo(index);
-            if (index.parent().isValid())
-                m_view->expand(index.parent());
-            return IFindSupport::Found;
-        }
-        return IFindSupport::NotFound;
-    }
-
-    void replace(const QString &before, const QString &after,
-        Find::FindFlags findFlags)
-    {
-        Q_UNUSED(before)
-        Q_UNUSED(after)
-        Q_UNUSED(findFlags)
-    }
-
-    bool replaceStep(const QString &before, const QString &after,
-        Find::FindFlags findFlags)
-    {
-        Q_UNUSED(before)
-        Q_UNUSED(after)
-        Q_UNUSED(findFlags)
-        return false;
-    }
-
-    int replaceAll(const QString &before, const QString &after,
-        Find::FindFlags findFlags)
-    {
-        Q_UNUSED(before)
-        Q_UNUSED(after)
-        Q_UNUSED(findFlags)
-        return 0;
-    }
-
-private:
-    SearchResultTreeView *m_view;
-    QModelIndex m_incrementalFindStart;
-    bool m_incrementalWrappedState;
-};
-
 } // Internal
 } // Find
 
@@ -208,7 +78,10 @@ using namespace Find::Internal;
 SearchResultWidget::SearchResultWidget(QWidget *parent) :
     QWidget(parent),
     m_count(0),
-    m_isShowingReplaceUI(false)
+    m_sizeWarningActive(false),
+    m_sizeWarningOverridden(false),
+    m_isShowingReplaceUI(false),
+    m_searchAgainSupported(false)
 {
     QVBoxLayout *layout = new QVBoxLayout(this);
     layout->setMargin(0);
@@ -226,18 +99,34 @@ SearchResultWidget::SearchResultWidget(QWidget *parent) :
     QHBoxLayout *topLayout = new QHBoxLayout(topWidget);
     topLayout->setMargin(2);
     topWidget->setLayout(topLayout);
+    layout->addWidget(topWidget);
+
+    m_messageWidget = new QFrame;
+    pal.setColor(QPalette::Window, QColor(255, 255, 225));
+    pal.setColor(QPalette::WindowText, Qt::red);
+    m_messageWidget->setPalette(pal);
+    m_messageWidget->setFrameStyle(QFrame::Panel | QFrame::Raised);
+    m_messageWidget->setLineWidth(1);
+    m_messageWidget->setAutoFillBackground(true);
+    QHBoxLayout *messageLayout = new QHBoxLayout(m_messageWidget);
+    messageLayout->setMargin(2);
+    m_messageWidget->setLayout(messageLayout);
+    QLabel *messageLabel = new QLabel(tr("Search was canceled."));
+    messageLabel->setPalette(pal);
+    messageLayout->addWidget(messageLabel);
+    layout->addWidget(m_messageWidget);
+    m_messageWidget->setVisible(false);
 
     m_searchResultTreeView = new Internal::SearchResultTreeView(this);
     m_searchResultTreeView->setFrameStyle(QFrame::NoFrame);
     m_searchResultTreeView->setAttribute(Qt::WA_MacShowFocusRect, false);
     Aggregation::Aggregate * agg = new Aggregation::Aggregate;
     agg->add(m_searchResultTreeView);
-    agg->add(new SearchResultFindSupport(m_searchResultTreeView));
-
-    layout->addWidget(topWidget);
+    agg->add(new TreeViewFind(m_searchResultTreeView,
+                              ItemDataRoles::ResultLineRole));
     layout->addWidget(m_searchResultTreeView);
 
-    m_infoBarDisplay.setTarget(layout, 1);
+    m_infoBarDisplay.setTarget(layout, 2);
     m_infoBarDisplay.setInfoBar(&m_infoBar);
 
     m_descriptionContainer = new QWidget(topWidget);
@@ -256,21 +145,30 @@ SearchResultWidget::SearchResultWidget(QWidget *parent) :
     m_cancelButton->setText(tr("Cancel"));
     m_cancelButton->setToolButtonStyle(Qt::ToolButtonTextOnly);
     connect(m_cancelButton, SIGNAL(clicked()), this, SLOT(cancel()));
+    m_searchAgainButton = new QToolButton(topWidget);
+    m_searchAgainButton->setToolTip(tr("Repeat the search with same parameters"));
+    m_searchAgainButton->setText(tr("Search again"));
+    m_searchAgainButton->setToolButtonStyle(Qt::ToolButtonTextOnly);
+    m_searchAgainButton->setVisible(false);
+    connect(m_searchAgainButton, SIGNAL(clicked()), this, SLOT(searchAgain()));
 
     m_replaceLabel = new QLabel(tr("Replace with:"), topWidget);
     m_replaceTextEdit = new WideEnoughLineEdit(topWidget);
     m_replaceTextEdit->setMinimumWidth(120);
+    m_replaceTextEdit->setEnabled(false);
+    m_replaceTextEdit->setTabOrder(m_replaceTextEdit, m_searchResultTreeView);
     m_replaceButton = new QToolButton(topWidget);
     m_replaceButton->setToolTip(tr("Replace all occurrences"));
     m_replaceButton->setText(tr("Replace"));
     m_replaceButton->setToolButtonStyle(Qt::ToolButtonTextOnly);
-    m_replaceTextEdit->setTabOrder(m_replaceTextEdit, m_searchResultTreeView);
+    m_replaceButton->setEnabled(false);
 
     m_matchesFoundLabel = new QLabel(topWidget);
     updateMatchesFoundLabel();
 
     topLayout->addWidget(m_descriptionContainer);
     topLayout->addWidget(m_cancelButton);
+    topLayout->addWidget(m_searchAgainButton);
     topLayout->addWidget(m_replaceLabel);
     topLayout->addWidget(m_replaceTextEdit);
     topLayout->addWidget(m_replaceButton);
@@ -287,6 +185,12 @@ SearchResultWidget::SearchResultWidget(QWidget *parent) :
     connect(m_replaceButton, SIGNAL(clicked()), this, SLOT(handleReplaceButton()));
 }
 
+SearchResultWidget::~SearchResultWidget()
+{
+    if (m_sizeWarningActive)
+        cancelAfterSizeWarning();
+}
+
 void SearchResultWidget::setInfo(const QString &label, const QString &toolTip, const QString &term)
 {
     m_label->setText(label);
@@ -315,9 +219,10 @@ void SearchResultWidget::addResults(const QList<SearchResultItem> &items, Search
     bool firstItems = (m_count == 0);
     m_count += items.size();
     m_searchResultTreeView->addResults(items, mode);
+    updateMatchesFoundLabel();
     if (firstItems) {
         if (showWarningMessage()) {
-            Core::InfoBarEntry info("warninglabel", tr("This change cannot be undone."));
+            Core::InfoBarEntry info(QLatin1String(UNDO_WARNING_ID), tr("This change cannot be undone."));
             info.setCustomButtonInfo(tr("Do not warn again"), this, SLOT(hideNoUndoWarning()));
             m_infoBar.addInfo(info);
         }
@@ -332,10 +237,21 @@ void SearchResultWidget::addResults(const QList<SearchResultItem> &items, Search
         }
         m_searchResultTreeView->selectionModel()->select(m_searchResultTreeView->model()->index(0, 0, QModelIndex()), QItemSelectionModel::Select);
         emit navigateStateChanged();
+    } else if (m_count > SEARCHRESULT_WARNING_LIMIT && !m_sizeWarningOverridden && !m_sizeWarningActive) {
+        m_sizeWarningActive = true;
+        emit paused(true);
+        Core::InfoBarEntry info(QLatin1String(SIZE_WARNING_ID),
+                                tr("The search resulted in more than %n items, do you still want to continue?",
+                                0, SEARCHRESULT_WARNING_LIMIT));
+        info.setCancelButtonInfo(tr("Cancel"), this, SLOT(cancelAfterSizeWarning()));
+        info.setCustomButtonInfo(tr("Continue"), this, SLOT(continueAfterSizeWarning()));
+        m_infoBar.addInfo(info);
+        emit requestPopup(false/*no focus*/);
     }
-    updateMatchesFoundLabel();
 }
 
+
+
 int SearchResultWidget::count() const
 {
     return m_count;
@@ -444,31 +360,73 @@ void SearchResultWidget::goToPrevious()
     }
 }
 
-void SearchResultWidget::finishSearch()
-{
-    m_replaceTextEdit->setEnabled(m_count > 0);
-    m_replaceButton->setEnabled(m_count > 0);
-    m_cancelButton->setVisible(false);
-}
-
-void SearchResultWidget::clear()
+void SearchResultWidget::restart()
 {
     m_replaceTextEdit->setEnabled(false);
     m_replaceButton->setEnabled(false);
-    m_replaceTextEdit->clear();
     m_searchResultTreeView->clear();
     m_count = 0;
-    m_label->setVisible(false);
-    m_searchTerm->setVisible(false);
-    m_cancelButton->setVisible(false);
+    if (m_sizeWarningActive)
+        m_infoBar.removeInfo(QLatin1String(SIZE_WARNING_ID));
+    m_sizeWarningActive = false;
+    m_sizeWarningOverridden = false;
+    m_cancelButton->setVisible(true);
+    m_searchAgainButton->setVisible(false);
+    m_messageWidget->setVisible(false);
     updateMatchesFoundLabel();
-    m_infoBar.clear();
+    emit restarted();
+}
+
+void SearchResultWidget::setSearchAgainSupported(bool supported)
+{
+    m_searchAgainSupported = supported;
+    m_searchAgainButton->setVisible(supported && !m_cancelButton->isVisible());
+}
+
+void SearchResultWidget::setSearchAgainEnabled(bool enabled)
+{
+    m_searchAgainButton->setEnabled(enabled);
+}
+
+void SearchResultWidget::finishSearch(bool canceled)
+{
+    if (m_sizeWarningActive)
+        m_infoBar.removeInfo(QLatin1String(SIZE_WARNING_ID));
+    m_sizeWarningActive = false;
+    m_sizeWarningOverridden = false;
+    m_replaceTextEdit->setEnabled(m_count > 0);
+    m_replaceButton->setEnabled(m_count > 0);
+    m_cancelButton->setVisible(false);
+    m_messageWidget->setVisible(canceled);
+    m_searchAgainButton->setVisible(m_searchAgainSupported);
+}
+
+void SearchResultWidget::sendRequestPopup()
+{
+    emit requestPopup(true/*focus*/);
 }
 
 void SearchResultWidget::hideNoUndoWarning()
 {
     setShowWarningMessage(false);
-    m_infoBar.clear();
+    m_infoBar.removeInfo(QLatin1String(UNDO_WARNING_ID));
+}
+
+void SearchResultWidget::continueAfterSizeWarning()
+{
+    m_sizeWarningOverridden = true;
+    m_sizeWarningActive = false;
+    m_infoBar.removeInfo(QLatin1String(SIZE_WARNING_ID));
+    emit paused(false);
+}
+
+void SearchResultWidget::cancelAfterSizeWarning()
+{
+    m_infoBar.removeInfo(QLatin1String(SIZE_WARNING_ID));
+    m_sizeWarningOverridden = true;
+    m_sizeWarningActive = false;
+    emit cancelled();
+    emit paused(false);
 }
 
 void SearchResultWidget::handleJumpToSearchResult(const SearchResultItem &item)
@@ -489,15 +447,23 @@ void SearchResultWidget::handleReplaceButton()
 void SearchResultWidget::cancel()
 {
     m_cancelButton->setVisible(false);
-    emit cancelled();
+    if (m_sizeWarningActive)
+        cancelAfterSizeWarning();
+    else
+        emit cancelled();
+}
+
+void SearchResultWidget::searchAgain()
+{
+    emit searchAgainRequested();
 }
 
 bool SearchResultWidget::showWarningMessage() const
 {
-    if (m_dontAskAgainGroup.isEmpty())
+    if (m_dontAskAgainGroup.isEmpty() || m_infoBar.containsInfo(QLatin1String(UNDO_WARNING_ID)))
         return false;
     // read settings
-    QSettings *settings = Core::ICore::instance()->settings();
+    QSettings *settings = Core::ICore::settings();
     settings->beginGroup(m_dontAskAgainGroup);
     settings->beginGroup(QLatin1String("Rename"));
     const bool showWarningMessage = settings->value(QLatin1String("ShowWarningMessage"), true).toBool();
@@ -509,7 +475,7 @@ bool SearchResultWidget::showWarningMessage() const
 void SearchResultWidget::setShowWarningMessage(bool showWarningMessage)
 {
     // write to settings
-    QSettings *settings = Core::ICore::instance()->settings();
+    QSettings *settings = Core::ICore::settings();
     settings->beginGroup(m_dontAskAgainGroup);
     settings->beginGroup(QLatin1String("Rename"));
     settings->setValue(QLatin1String("ShowWarningMessage"), showWarningMessage);
