@@ -1,20 +1,19 @@
-/**************************************************************************
+/****************************************************************************
 **
-** This file is part of Qt Creator
+** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
-** Copyright (c) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** This file is part of Qt Creator.
 **
-** Contact: Nokia Corporation (qt-info@nokia.com)
-**
-** Commercial Usage
-**
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
-**
 ** Alternatively, this file may be used under the terms of the GNU Lesser
 ** General Public License version 2.1 as published by the Free Software
 ** Foundation and appearing in the file LICENSE.LGPL included in the
@@ -22,58 +21,61 @@
 ** ensure the GNU Lesser General Public License version 2.1 requirements
 ** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** If you are unsure which license is appropriate for your use, please
-** contact the sales department at http://qt.nokia.com/contact.
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
+** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-**************************************************************************/
+****************************************************************************/
 
 #include <qglobal.h>
 
-namespace Locator {
-struct FilterEntry;
-}
-
-QT_BEGIN_NAMESPACE
-unsigned int qHash(const Locator::FilterEntry &entry);
-QT_END_NAMESPACE
-
 #include "locatorwidget.h"
 #include "locatorplugin.h"
 #include "locatorconstants.h"
+#include "ilocatorfilter.h"
 
 #include <extensionsystem/pluginmanager.h>
+#include <coreplugin/coreconstants.h>
 #include <coreplugin/icore.h>
 #include <coreplugin/modemanager.h>
+#include <coreplugin/actionmanager/actionmanager.h>
+#include <coreplugin/actionmanager/command.h>
+#include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/coreconstants.h>
 #include <coreplugin/fileiconprovider.h>
-#include <utils/fancylineedit.h>
+#include <utils/filterlineedit.h>
 #include <utils/qtcassert.h>
+#include <utils/runextensions.h>
 
-#include <QtCore/QFileInfo>
-#include <QtCore/QFile>
-#include <QtCore/QTimer>
-#include <QtCore/QSettings>
-#include <QtGui/QAction>
-#include <QtGui/QApplication>
-#include <QtGui/QContextMenuEvent>
-#include <QtGui/QHBoxLayout>
-#include <QtGui/QHeaderView>
-#include <QtGui/QKeyEvent>
-#include <QtGui/QLabel>
-#include <QtGui/QLineEdit>
-#include <QtGui/QMenu>
-#include <QtGui/QMouseEvent>
-#include <QtGui/QPushButton>
-#include <QtGui/QScrollBar>
-#include <QtGui/QTreeView>
+#include <QtConcurrentRun>
+#include <QColor>
+#include <QFileInfo>
+#include <QFile>
+#include <QTimer>
+#include <QSettings>
+#include <QEvent>
+#include <QAction>
+#include <QApplication>
+#include <QContextMenuEvent>
+#include <QHBoxLayout>
+#include <QHeaderView>
+#include <QKeyEvent>
+#include <QLabel>
+#include <QLineEdit>
+#include <QMenu>
+#include <QMouseEvent>
+#include <QPushButton>
+#include <QScrollBar>
+#include <QTreeView>
+#include <QToolTip>
 
-Q_DECLARE_METATYPE(Locator::ILocatorFilter*);
-Q_DECLARE_METATYPE(Locator::FilterEntry);
+Q_DECLARE_METATYPE(Locator::ILocatorFilter*)
+Q_DECLARE_METATYPE(Locator::FilterEntry)
 
 namespace Locator {
 namespace Internal {
 
-/*! A model to represent the Locator results. */
+/* A model to represent the Locator results. */
 class LocatorModel : public QAbstractListModel
 {
 public:
@@ -102,26 +104,51 @@ public:
     void updatePreferredSize();
     QSize preferredSize() const { return m_preferredSize; }
 
+#if defined(Q_OS_WIN)
+    void focusOutEvent (QFocusEvent * event)  {
+        if (event->reason() == Qt::ActiveWindowFocusReason)
+            hide();
+    }
+#endif
+
+    void next() {
+        int index = currentIndex().row();
+        ++index;
+        if (index >= model()->rowCount(QModelIndex())) {
+            // wrap
+            index = 0;
+        }
+        setCurrentIndex(model()->index(index, 0));
+    }
+
+    void previous() {
+        int index = currentIndex().row();
+        --index;
+        if (index < 0) {
+            // wrap
+            index = model()->rowCount(QModelIndex()) - 1;
+        }
+        setCurrentIndex(model()->index(index, 0));
+    }
+
 private:
     QSize m_preferredSize;
 };
 
 } // namespace Internal
+
+uint qHash(const FilterEntry &entry)
+{
+    if (entry.internalData.canConvert(QVariant::String))
+        return QT_PREPEND_NAMESPACE(qHash)(entry.internalData.toString());
+    return QT_PREPEND_NAMESPACE(qHash)(entry.internalData.constData());
+}
+
 } // namespace Locator
 
 using namespace Locator;
 using namespace Locator::Internal;
 
-QT_BEGIN_NAMESPACE
-uint qHash(const FilterEntry &entry)
-{
-    if (entry.internalData.canConvert(QVariant::String))
-        return qHash(entry.internalData.toString());
-    return qHash(entry.internalData.constData());
-}
-QT_END_NAMESPACE
-
-
 // =========== LocatorModel ===========
 
 int LocatorModel::rowCount(const QModelIndex & /* parent */) const
@@ -145,22 +172,29 @@ QVariant LocatorModel::data(const QModelIndex &index, int role) const
     if (!index.isValid() || index.row() >= mEntries.size())
         return QVariant();
 
-    if (role == Qt::DisplayRole  || role == Qt::ToolTipRole) {
+    if (role == Qt::DisplayRole) {
         if (index.column() == 0) {
             return mEntries.at(index.row()).displayName;
         } else if (index.column() == 1) {
             return mEntries.at(index.row()).extraInfo;
         }
+    } else if (role == Qt::ToolTipRole) {
+        if (mEntries.at(index.row()).extraInfo.isEmpty())
+            return QVariant(mEntries.at(index.row()).displayName);
+        else
+            return QVariant(mEntries.at(index.row()).displayName
+                            + QLatin1String("\n\n") + mEntries.at(index.row()).extraInfo);
     } else if (role == Qt::DecorationRole && index.column() == 0) {
         FilterEntry &entry = mEntries[index.row()];
         if (entry.resolveFileIcon && entry.displayIcon.isNull()) {
             entry.resolveFileIcon = false;
-            entry.displayIcon =
-                 Core::FileIconProvider::instance()->icon(QFileInfo(entry.internalData.toString()));
+            QString path = entry.internalData.toString();
+            Core::EditorManager::splitLineNumber(&path);
+            entry.displayIcon = Core::FileIconProvider::instance()->icon(QFileInfo(path));
         }
         return entry.displayIcon;
     } else if (role == Qt::ForegroundRole && index.column() == 1) {
-        return Qt::darkGray;
+        return QColor(Qt::darkGray);
     } else if (role == Qt::UserRole) {
         return qVariantFromValue(mEntries.at(index.row()));
     }
@@ -173,30 +207,6 @@ void LocatorModel::setEntries(const QList<FilterEntry> &entries)
     mEntries = entries;
     reset();
 }
-#if 0
-void LocatorModel::setDisplayCount(int count)
-{
-    // TODO: This method is meant to be used for increasing the number of items displayed at the
-    // user's request. There is however no way yet for the user to request this.
-    if (count == mDisplayCount)
-        return;
-
-    const int displayedOld = qMin(mDisplayCount, mEntries.size());
-    const int displayedNew = qMin(count, mEntries.size());
-
-    if (displayedNew < displayedOld)
-        beginRemoveRows(QModelIndex(), displayedNew - 1, displayedOld - 1);
-    else if (displayedNew > displayedOld)
-        beginInsertRows(QModelIndex(), displayedOld - 1, displayedNew - 1);
-
-    mDisplayCount = count;
-
-    if (displayedNew < displayedOld)
-        endRemoveRows();
-    else if (displayedNew > displayedOld)
-        endInsertRows();
-}
-#endif
 
 // =========== CompletionList ===========
 
@@ -211,49 +221,34 @@ CompletionList::CompletionList(QWidget *parent)
     // This is too slow when done on all results
     //header()->setResizeMode(QHeaderView::ResizeToContents);
     setWindowFlags(Qt::ToolTip);
+#ifdef Q_OS_MAC
+    if (horizontalScrollBar())
+        horizontalScrollBar()->setAttribute(Qt::WA_MacMiniSize);
+    if (verticalScrollBar())
+        verticalScrollBar()->setAttribute(Qt::WA_MacMiniSize);
+#endif
 }
 
 void CompletionList::updatePreferredSize()
 {
-    //header()->setStretchLastSection(false);
-    //updateGeometries();
-
     const QStyleOptionViewItem &option = viewOptions();
     const QSize shint = itemDelegate()->sizeHint(option, model()->index(0, 0));
-#if 0
-    const int visibleItems = model()->rowCount();
 
-    // TODO: Look into enabling preferred height as well. Current problem is that this doesn't
-    // work nicely from the user perspective if the list is popping up instead of down.
-    //const int h = shint.height() * visibleItems;
-
-    const QScrollBar *vscroll = verticalScrollBar();
-    int preferredWidth = header()->length() + frameWidth() * 2
-                         + (vscroll->isVisibleTo(this) ? vscroll->width() : 0);
-    const int diff = preferredWidth - width();
-
-    // Avoid resizing the list widget when there are no results or when the preferred size is
-    // only a little smaller than our current size
-    if (visibleItems == 0 || (diff > -100 && diff < 0))
-        preferredWidth = width();
-#endif
-
-    m_preferredSize = QSize(730, //qMax(600, preferredWidth),
-                            shint.height() * 17 + frameWidth() * 2);
-    //header()->setStretchLastSection(true);
+    m_preferredSize = QSize(730, shint.height() * 17 + frameWidth() * 2);
 }
 
-
 // =========== LocatorWidget ===========
 
 LocatorWidget::LocatorWidget(LocatorPlugin *qop) :
-     m_locatorPlugin(qop),
-     m_locatorModel(new LocatorModel(this)),
-     m_completionList(new CompletionList(this)),
-     m_filterMenu(new QMenu(this)),
-     m_refreshAction(new QAction(tr("Refresh"), this)),
-     m_configureAction(new QAction(tr("Configure..."), this)),
-     m_fileLineEdit(new Utils::FancyLineEdit)
+    m_locatorPlugin(qop),
+    m_locatorModel(new LocatorModel(this)),
+    m_completionList(new CompletionList(this)),
+    m_filterMenu(new QMenu(this)),
+    m_refreshAction(new QAction(tr("Refresh"), this)),
+    m_configureAction(new QAction(tr("Configure..."), this)),
+    m_fileLineEdit(new Utils::FilterLineEdit),
+    m_updateRequested(false),
+    m_acceptRequested(false)
 {
     // Explicitly hide the completion list popup.
     m_completionList->hide();
@@ -272,12 +267,14 @@ LocatorWidget::LocatorWidget(LocatorPlugin *qop) :
     layout->setMargin(0);
     layout->addWidget(m_fileLineEdit);
 
-    setWindowIcon(QIcon(":/locator/images/locator.png"));
-    QPixmap image(Core::Constants::ICON_MAGNIFIER);
-    m_fileLineEdit->setPixmap(image);
-    m_fileLineEdit->setUseLayoutDirection(true);
-    m_fileLineEdit->setHintText(tr("Type to locate"));
+    setWindowIcon(QIcon(QLatin1String(":/locator/images/locator.png")));
+    const QPixmap image = QPixmap(QLatin1String(Core::Constants::ICON_MAGNIFIER));
+    m_fileLineEdit->setButtonPixmap(Utils::FancyLineEdit::Left, image);
+    m_fileLineEdit->setButtonToolTip(Utils::FancyLineEdit::Left, tr("Options"));
     m_fileLineEdit->setFocusPolicy(Qt::ClickFocus);
+    m_fileLineEdit->setButtonVisible(Utils::FancyLineEdit::Left, true);
+    // We set click focus since otherwise you will always get two popups
+    m_fileLineEdit->setButtonFocusPolicy(Utils::FancyLineEdit::Left, Qt::ClickFocus);
     m_fileLineEdit->setAttribute(Qt::WA_MacShowFocusRect, false);
 
     m_fileLineEdit->installEventFilter(this);
@@ -291,30 +288,66 @@ LocatorWidget::LocatorWidget(LocatorPlugin *qop) :
     m_filterMenu->addAction(m_refreshAction);
     m_filterMenu->addAction(m_configureAction);
 
-    m_fileLineEdit->setMenu( m_filterMenu);
+    m_fileLineEdit->setButtonMenu(Utils::FancyLineEdit::Left, m_filterMenu);
 
     connect(m_refreshAction, SIGNAL(triggered()), m_locatorPlugin, SLOT(refresh()));
     connect(m_configureAction, SIGNAL(triggered()), this, SLOT(showConfigureDialog()));
-    connect(m_fileLineEdit, SIGNAL(textEdited(const QString&)),
+    connect(m_fileLineEdit, SIGNAL(textChanged(QString)),
         this, SLOT(showPopup()));
     connect(m_completionList, SIGNAL(activated(QModelIndex)),
-            this, SLOT(acceptCurrentEntry()));
+            this, SLOT(scheduleAcceptCurrentEntry()));
+
+    m_entriesWatcher = new QFutureWatcher<FilterEntry>(this);
+    connect(m_entriesWatcher, SIGNAL(finished()), SLOT(updateEntries()));
+
+    m_showPopupTimer = new QTimer(this);
+    m_showPopupTimer->setInterval(100);
+    m_showPopupTimer->setSingleShot(true);
+    connect(m_showPopupTimer, SIGNAL(timeout()), SLOT(showPopupNow()));
 }
 
-bool LocatorWidget::isShowingTypeHereMessage() const
+void LocatorWidget::setPlaceholderText(const QString &text)
 {
-    return m_fileLineEdit->isShowingHintText();
+    m_fileLineEdit->setPlaceholderText(text);
 }
 
 void LocatorWidget::updateFilterList()
 {
     m_filterMenu->clear();
+
+    // update actions and menu
+    QMap<QString, QAction *> actionCopy = m_filterActionMap;
+    m_filterActionMap.clear();
+    // register new actions, update existent
     foreach (ILocatorFilter *filter, m_locatorPlugin->filters()) {
-        if (!filter->shortcutString().isEmpty() && !filter->isHidden()) {
-            QAction *action = m_filterMenu->addAction(filter->trName(), this, SLOT(filterSelected()));
+        if (filter->shortcutString().isEmpty() || filter->isHidden())
+            continue;
+        Core::Id locatorId = Core::Id(QLatin1String("Locator.") + filter->id());
+        QAction *action = 0;
+        Core::Command *cmd = 0;
+        if (!actionCopy.contains(filter->id())) {
+            // register new action
+            action = new QAction(filter->displayName(), this);
+            cmd = Core::ActionManager::registerAction(action, locatorId,
+                               Core::Context(Core::Constants::C_GLOBAL));
+            cmd->setAttribute(Core::Command::CA_UpdateText);
+            connect(action, SIGNAL(triggered()), this, SLOT(filterSelected()));
             action->setData(qVariantFromValue(filter));
+        } else {
+            action = actionCopy.take(filter->id());
+            action->setText(filter->displayName());
+            cmd = Core::ActionManager::command(locatorId);
         }
+        m_filterActionMap.insert(filter->id(), action);
+        m_filterMenu->addAction(cmd->action());
     }
+
+    // unregister actions that are deleted now
+    foreach (const QString &id, actionCopy.keys()) {
+        Core::ActionManager::unregisterAction(actionCopy.value(id), Core::Id(QLatin1String("Locator.") + id));
+    }
+    qDeleteAll(actionCopy);
+
     m_filterMenu->addSeparator();
     m_filterMenu->addAction(m_refreshAction);
     m_filterMenu->addAction(m_configureAction);
@@ -323,6 +356,11 @@ void LocatorWidget::updateFilterList()
 bool LocatorWidget::eventFilter(QObject *obj, QEvent *event)
 {
     if (obj == m_fileLineEdit && event->type() == QEvent::KeyPress) {
+        if (m_possibleToolTipRequest)
+            m_possibleToolTipRequest = false;
+        if (QToolTip::isVisible())
+            QToolTip::hideText();
+
         QKeyEvent *keyEvent = static_cast<QKeyEvent *>(event);
         switch (keyEvent->key()) {
         case Qt::Key_Up:
@@ -334,29 +372,77 @@ bool LocatorWidget::eventFilter(QObject *obj, QEvent *event)
             return true;
         case Qt::Key_Enter:
         case Qt::Key_Return:
-            acceptCurrentEntry();
+            scheduleAcceptCurrentEntry();
             return true;
         case Qt::Key_Escape:
             m_completionList->hide();
             return true;
+        case Qt::Key_Tab:
+            m_completionList->next();
+            return true;
+        case Qt::Key_Backtab:
+            m_completionList->previous();
+            return true;
+        case Qt::Key_Alt:
+            if (keyEvent->modifiers() == Qt::AltModifier) {
+                m_possibleToolTipRequest = true;
+                return true;
+            }
+            break;
         default:
             break;
         }
+    } else if (obj == m_fileLineEdit && event->type() == QEvent::KeyRelease) {
+        QKeyEvent *keyEvent = static_cast<QKeyEvent *>(event);
+        if (m_possibleToolTipRequest) {
+            m_possibleToolTipRequest = false;
+            if (m_completionList->isVisible()
+                    && (keyEvent->key() == Qt::Key_Alt)
+                    && (keyEvent->modifiers() == Qt::NoModifier)) {
+                const QModelIndex index = m_completionList->currentIndex();
+                if (index.isValid()) {
+                    QToolTip::showText(m_completionList->pos() + m_completionList->visualRect(index).topRight(),
+                                       m_locatorModel->data(index, Qt::ToolTipRole).toString());
+                    return true;
+                }
+            }
+        }
     } else if (obj == m_fileLineEdit && event->type() == QEvent::FocusOut) {
-        m_completionList->hide();
+#if defined(Q_OS_WIN)
+        QFocusEvent *fev = static_cast<QFocusEvent*>(event);
+        if (fev->reason() != Qt::ActiveWindowFocusReason ||
+            (fev->reason() == Qt::ActiveWindowFocusReason && !m_completionList->isActiveWindow()))
+#endif
+            m_completionList->hide();
     } else if (obj == m_fileLineEdit && event->type() == QEvent::FocusIn) {
-        showPopup();
+        showPopupNow();
     } else if (obj == this && event->type() == QEvent::ShortcutOverride) {
         QKeyEvent *ke = static_cast<QKeyEvent *>(event);
-        if (ke->key() == Qt::Key_Escape && !ke->modifiers()) {
-            event->accept();
-            QTimer::singleShot(0, Core::ModeManager::instance(), SLOT(setFocusToCurrentMode()));
-            return true;
+        switch (ke->key()) {
+        case Qt::Key_Escape:
+            if (!ke->modifiers()) {
+                event->accept();
+                QTimer::singleShot(0, this, SLOT(setFocusToCurrentMode()));
+                return true;
+            }
+        case Qt::Key_Alt:
+            if (ke->modifiers() == Qt::AltModifier) {
+                event->accept();
+                return true;
+            }
+            break;
+        default:
+            break;
         }
     }
     return QWidget::eventFilter(obj, event);
 }
 
+void LocatorWidget::setFocusToCurrentMode()
+{
+    Core::ModeManager::setFocusToCurrentMode();
+}
+
 void LocatorWidget::showCompletionList()
 {
     const int border = m_completionList->frameWidth();
@@ -368,25 +454,35 @@ void LocatorWidget::showCompletionList()
 
 void LocatorWidget::showPopup()
 {
-    updateCompletionList(m_fileLineEdit->typedText());
+    m_updateRequested = true;
+    m_showPopupTimer->start();
+}
+
+void LocatorWidget::showPopupNow()
+{
+    m_showPopupTimer->stop();
+    updateCompletionList(m_fileLineEdit->text());
     showCompletionList();
 }
 
 QList<ILocatorFilter*> LocatorWidget::filtersFor(const QString &text, QString &searchText)
 {
     QList<ILocatorFilter*> filters = m_locatorPlugin->filters();
-    int whiteSpace = text.indexOf(" ");
+    const int whiteSpace = text.indexOf(QLatin1Char(' '));
     QString prefix;
     if (whiteSpace >= 0)
         prefix = text.left(whiteSpace);
     if (!prefix.isEmpty()) {
         prefix = prefix.toLower();
+        QList<ILocatorFilter *> prefixFilters;
         foreach (ILocatorFilter *filter, filters) {
             if (prefix == filter->shortcutString()) {
                 searchText = text.mid(whiteSpace+1);
-                return QList<ILocatorFilter*>() << filter;
+                prefixFilters << filter;
             }
         }
+        if (!prefixFilters.isEmpty())
+            return prefixFilters;
     }
     searchText = text;
     QList<ILocatorFilter*> activeFilters;
@@ -396,22 +492,48 @@ QList<ILocatorFilter*> LocatorWidget::filtersFor(const QString &text, QString &s
     return activeFilters;
 }
 
-void LocatorWidget::updateCompletionList(const QString &text)
+static void filter_helper(QFutureInterface<Locator::FilterEntry> &entries, QList<ILocatorFilter *> filters, QString searchText)
 {
-    QString searchText;
-    const QList<ILocatorFilter*> filters = filtersFor(text, searchText);
     QSet<FilterEntry> alreadyAdded;
     const bool checkDuplicates = (filters.size() > 1);
-    QList<FilterEntry> entries;
     foreach (ILocatorFilter *filter, filters) {
-        foreach (const FilterEntry &entry, filter->matchesFor(searchText)) {
+        if (entries.isCanceled())
+            break;
+
+        foreach (const FilterEntry &entry, filter->matchesFor(entries, searchText)) {
             if (checkDuplicates && alreadyAdded.contains(entry))
                 continue;
-            entries.append(entry);
+            entries.reportResult(entry);
             if (checkDuplicates)
                 alreadyAdded.insert(entry);
         }
     }
+}
+
+void LocatorWidget::updateCompletionList(const QString &text)
+{
+    m_updateRequested = true;
+    QString searchText;
+    const QList<ILocatorFilter*> filters = filtersFor(text, searchText);
+
+    // cancel the old future
+    m_entriesWatcher->future().cancel();
+    m_entriesWatcher->future().waitForFinished();
+
+    QFuture<FilterEntry> future = QtConcurrent::run(filter_helper, filters, searchText);
+    m_entriesWatcher->setFuture(future);
+}
+
+void LocatorWidget::updateEntries()
+{
+    m_updateRequested = false;
+    if (m_entriesWatcher->future().isCanceled()) {
+        // reset to usable state
+        m_acceptRequested = false;
+        return;
+    }
+
+    const QList<FilterEntry> entries = m_entriesWatcher->future().results();
     m_locatorModel->setEntries(entries);
     if (m_locatorModel->rowCount() > 0) {
         m_completionList->setCurrentIndex(m_locatorModel->index(0, 0));
@@ -419,10 +541,24 @@ void LocatorWidget::updateCompletionList(const QString &text)
 #if 0
     m_completionList->updatePreferredSize();
 #endif
+    if (m_acceptRequested)
+        acceptCurrentEntry();
+}
+
+void LocatorWidget::scheduleAcceptCurrentEntry()
+{
+    if (m_updateRequested) {
+        // don't just accept the selected entry, since the list is not up to date
+        // accept will be called after the update finished
+        m_acceptRequested = true;
+    } else {
+        acceptCurrentEntry();
+    }
 }
 
 void LocatorWidget::acceptCurrentEntry()
 {
+    m_acceptRequested = false;
     if (!m_completionList->isVisible())
         return;
     const QModelIndex index = m_completionList->currentIndex();
@@ -435,13 +571,12 @@ void LocatorWidget::acceptCurrentEntry()
 
 void LocatorWidget::show(const QString &text, int selectionStart, int selectionLength)
 {
-    m_fileLineEdit->hideHintText();
     if (!text.isEmpty())
         m_fileLineEdit->setText(text);
     if (!m_fileLineEdit->hasFocus())
         m_fileLineEdit->setFocus();
     else
-        showPopup();
+        showPopupNow();
 
     if (selectionStart >= 0) {
         m_fileLineEdit->setSelection(selectionStart, selectionLength);
@@ -464,13 +599,13 @@ void LocatorWidget::filterSelected()
     if (!currentText.isEmpty()) {
         searchText = currentText;
         foreach (ILocatorFilter *otherfilter, m_locatorPlugin->filters()) {
-            if (currentText.startsWith(otherfilter->shortcutString() + " ")) {
+            if (currentText.startsWith(otherfilter->shortcutString() + QLatin1Char(' '))) {
                 searchText = currentText.mid(otherfilter->shortcutString().length()+1);
                 break;
             }
         }
     }
-    show(filter->shortcutString() + " " + searchText,
+    show(filter->shortcutString() + QLatin1Char(' ') + searchText,
          filter->shortcutString().length() + 1,
          searchText.length());
     updateCompletionList(m_fileLineEdit->text());
@@ -484,6 +619,6 @@ void LocatorWidget::showEvent(QShowEvent *event)
 
 void LocatorWidget::showConfigureDialog()
 {
-    Core::ICore::instance()->showOptionsDialog(Constants::LOCATOR_CATEGORY,
-          Constants::FILTER_OPTIONS_PAGE);
+    Core::ICore::showOptionsDialog(QLatin1String(Core::Constants::SETTINGS_CATEGORY_CORE),
+          QLatin1String(Constants::FILTER_OPTIONS_PAGE));
 }
