@@ -1,34 +1,31 @@
-/**************************************************************************
+/****************************************************************************
 **
-** This file is part of Qt Creator
+** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
-**
-** Contact: Nokia Corporation (info@qt.nokia.com)
+** This file is part of Qt Creator.
 **
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** This file may be used under the terms of the GNU Lesser General Public
-** License version 2.1 as published by the Free Software Foundation and
-** appearing in the file LICENSE.LGPL included in the packaging of this file.
-** Please review the following information to ensure the GNU Lesser General
-** Public License version 2.1 requirements will be met:
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Nokia gives you certain additional
-** rights. These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** Other Usage
-**
-** Alternatively, this file may be used in accordance with the terms and
-** conditions contained in a signed written agreement between you and Nokia.
-**
-** If you have questions regarding the use of this file, please contact
-** Nokia at info@qt.nokia.com.
-**
-**************************************************************************/
+****************************************************************************/
 
 #include "plaintexteditmodifier.h"
 
@@ -36,10 +33,10 @@
 #include <extensionsystem/pluginmanager.h>
 #include <qmljs/qmljsmodelmanagerinterface.h>
 
-#include <QtGui/QPlainTextEdit>
-#include <QtGui/QUndoStack>
+#include <QPlainTextEdit>
+#include <QUndoStack>
 
-#include <QtCore/QDebug>
+#include <QDebug>
 
 using namespace Utils;
 using namespace QmlDesigner;
@@ -203,17 +200,9 @@ void PlainTextEditModifier::reactivateChangeSignals()
     }
 }
 
-namespace {
-static inline QmlJS::ModelManagerInterface *getModelManager()
-{
-    ExtensionSystem::PluginManager *pluginManager = ExtensionSystem::PluginManager::instance();
-    return pluginManager->getObject<QmlJS::ModelManagerInterface>();
-}
-}
-
 QmlJS::Snapshot NotIndentingTextEditModifier::getSnapshot() const
 {
-    QmlJS::ModelManagerInterface *modelManager = getModelManager();
+    QmlJS::ModelManagerInterface *modelManager = QmlJS::ModelManagerInterface::instance();
     if (modelManager)
         return modelManager->snapshot();
     else
@@ -223,7 +212,7 @@ QmlJS::Snapshot NotIndentingTextEditModifier::getSnapshot() const
 
 QStringList NotIndentingTextEditModifier::importPaths() const
 {
-    QmlJS::ModelManagerInterface *modelManager = getModelManager();
+    QmlJS::ModelManagerInterface *modelManager = QmlJS::ModelManagerInterface::instance();
     if (modelManager)
         return modelManager->importPaths();
     else
