@@ -1,51 +1,52 @@
-/**************************************************************************
+/****************************************************************************
 **
-** This file is part of Qt Creator
+** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
-**
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** This file is part of Qt Creator.
 **
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** This file may be used under the terms of the GNU Lesser General Public
-** License version 2.1 as published by the Free Software Foundation and
-** appearing in the file LICENSE.LGPL included in the packaging of this file.
-** Please review the following information to ensure the GNU Lesser General
-** Public License version 2.1 requirements will be met:
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Nokia gives you certain additional
-** rights. These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** Other Usage
-**
-** Alternatively, this file may be used in accordance with the terms and
-** conditions contained in a signed written agreement between you and Nokia.
-**
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
-**
-**************************************************************************/
+****************************************************************************/
 
 #include "qtversionmanager.h"
 
+#include "qtkitinformation.h"
 #include "qtversionfactory.h"
 
-#include <projectexplorer/debugginghelper.h>
+#include "qtsupportconstants.h"
+
 // only for legay restore
 #include <projectexplorer/projectexplorerconstants.h>
 #include <projectexplorer/toolchainmanager.h>
 #include <projectexplorer/gcctoolchain.h>
 
+#include <qtsupport/debugginghelper.h>
 
 #include <coreplugin/icore.h>
 #include <coreplugin/helpmanager.h>
 
 #include <extensionsystem/pluginmanager.h>
 
+#include <utils/filesystemwatcher.h>
 #include <utils/persistentsettings.h>
 #include <utils/qtcprocess.h>
 #include <utils/qtcassert.h>
@@ -53,28 +54,23 @@
 #    include <utils/winutils.h>
 #endif
 
-#include <QtCore/QFile>
-#include <QtCore/QSettings>
-#include <QtCore/QTextStream>
-#include <QtCore/QDir>
-#include <QtGui/QMainWindow>
+#include <QDebug>
+#include <QDir>
+#include <QFile>
+#include <QSettings>
+#include <QTextStream>
+#include <QTimer>
 
 #include <algorithm>
 
 using namespace QtSupport;
 using namespace QtSupport::Internal;
 
-using ProjectExplorer::DebuggingHelperLibrary;
-
 static const char QTVERSION_DATA_KEY[] = "QtVersion.";
 static const char QTVERSION_TYPE_KEY[] = "QtVersion.Type";
-static const char QTVERSION_COUNT_KEY[] = "QtVersion.Count";
-static const char OLDQTVERSION_COUNT_KEY[] = "QtVersion.Old.Count";
-static const char OLDQTVERSION_DATA_KEY[] = "QtVersion.Old.";
-static const char OLDQTVERSION_SDKSOURCE[] = "QtVersion.Old.SdkSource";
-static const char OLDQTVERSION_PATH[] = "QtVersion.Old.Path";
 static const char QTVERSION_FILE_VERSION_KEY[] = "Version";
-static const char QTVERSION_FILENAME[] = "/qtversion.xml";
+static const char QTVERSION_FILENAME[] = "/qtcreator/qtversion.xml";
+static const char QTVERSION_LEGACY_FILENAME[] = "/qtversion.xml"; // TODO: pre 2.6, remove later
 
 // legacy settings
 static const char QtVersionsSectionName[] = "QtVersions";
@@ -85,7 +81,7 @@ template<class T>
 static T *createToolChain(const QString &id)
 {
     QList<ProjectExplorer::ToolChainFactory *> factories =
-            ExtensionSystem::PluginManager::instance()->getObjects<ProjectExplorer::ToolChainFactory>();
+            ExtensionSystem::PluginManager::getObjects<ProjectExplorer::ToolChainFactory>();
     foreach (ProjectExplorer::ToolChainFactory *f, factories) {
        if (f->id() == id) {
            Q_ASSERT(f->canCreate());
@@ -95,11 +91,18 @@ static T *createToolChain(const QString &id)
     return 0;
 }
 
-static QString settingsFileName()
+static Utils::FileName globalSettingsFileName()
 {
-    ExtensionSystem::PluginManager *pm = ExtensionSystem::PluginManager::instance();
-    QFileInfo settingsLocation(pm->settings()->fileName());
-    return settingsLocation.absolutePath() + QLatin1String(QTVERSION_FILENAME);
+    QSettings *globalSettings = ExtensionSystem::PluginManager::globalSettings();
+    return Utils::FileName::fromString(QFileInfo(globalSettings->fileName()).absolutePath()
+                                       + QLatin1String(QTVERSION_FILENAME));
+}
+
+static Utils::FileName settingsFileName(const QString &path)
+{
+    QSettings *settings = ExtensionSystem::PluginManager::settings();
+    QFileInfo settingsLocation(settings->fileName());
+    return Utils::FileName::fromString(settingsLocation.absolutePath() + path);
 }
 
 
@@ -114,31 +117,54 @@ bool qtVersionNumberCompare(BaseQtVersion *a, BaseQtVersion *b)
 // --------------------------------------------------------------------------
 QtVersionManager *QtVersionManager::m_self = 0;
 
-QtVersionManager::QtVersionManager()
+QtVersionManager::QtVersionManager() :
+    m_configFileWatcher(0),
+    m_fileWatcherTimer(new QTimer(this)),
+    m_writer(0)
 {
     m_self = this;
     m_idcount = 1;
+
+    qRegisterMetaType<Utils::FileName>();
+
+    // Give the file a bit of time to settle before reading it...
+    m_fileWatcherTimer->setInterval(2000);
+    connect(m_fileWatcherTimer, SIGNAL(timeout()), SLOT(updateFromInstaller()));
 }
 
 void QtVersionManager::extensionsInitialized()
 {
     bool success = restoreQtVersions();
-    if (!success)
-        success = legacyRestore();
-    updateFromInstaller();
+    updateFromInstaller(false);
     if (!success) {
         // We did neither restore our settings or upgraded
         // in that case figure out if there's a qt in path
-        // and add it to the qt versions
+        // and add it to the Qt versions
         findSystemQt();
     }
 
-    updateSettings();
+    emit qtVersionsChanged(m_versions.keys(), QList<int>(), QList<int>());
     saveQtVersions();
+
+    const Utils::FileName configFileName = globalSettingsFileName();
+    if (configFileName.toFileInfo().exists()) {
+        m_configFileWatcher = new Utils::FileSystemWatcher(this);
+        connect(m_configFileWatcher, SIGNAL(fileChanged(QString)),
+                m_fileWatcherTimer, SLOT(start()));
+        m_configFileWatcher->addFile(configFileName.toString(),
+                                     Utils::FileSystemWatcher::WatchModifiedDate);
+    } // exists
+}
+
+bool QtVersionManager::delayedInitialize()
+{
+    updateDocumentation();
+    return true;
 }
 
 QtVersionManager::~QtVersionManager()
 {
+    delete m_writer;
     qDeleteAll(m_versions);
     m_versions.clear();
 }
@@ -150,11 +176,19 @@ QtVersionManager *QtVersionManager::instance()
 
 bool QtVersionManager::restoreQtVersions()
 {
-    ExtensionSystem::PluginManager *pm = ExtensionSystem::PluginManager::instance();
-    QList<QtVersionFactory *> factories = pm->getObjects<QtVersionFactory>();
+    QTC_ASSERT(!m_writer, return false);
+    m_writer = new Utils::PersistentSettingsWriter(settingsFileName(QLatin1String(QTVERSION_FILENAME)),
+                                                   QLatin1String("QtCreatorQtVersions"));
+
+    QList<QtVersionFactory *> factories = ExtensionSystem::PluginManager::getObjects<QtVersionFactory>();
 
     Utils::PersistentSettingsReader reader;
-    if (!reader.load(settingsFileName()))
+    Utils::FileName filename = settingsFileName(QLatin1String(QTVERSION_FILENAME));
+
+    // Read Qt Creator 2.5 qtversions.xml once:
+    if (!filename.toFileInfo().exists())
+        filename = settingsFileName(QLatin1String(QTVERSION_LEGACY_FILENAME));
+    if (!reader.load(filename))
         return false;
     QVariantMap data = reader.restoreValues();
 
@@ -163,15 +197,17 @@ bool QtVersionManager::restoreQtVersions()
     if (version < 1)
         return false;
 
-
-    int count = data.value(QLatin1String(QTVERSION_COUNT_KEY), 0).toInt();
-    for (int i = 0; i < count; ++i) {
-        const QString key = QString::fromLatin1(QTVERSION_DATA_KEY) + QString::number(i);
-        if (!data.contains(key))
-            break;
+    const QString keyPrefix = QLatin1String(QTVERSION_DATA_KEY);
+    foreach (const QString &key, data.keys()) {
+        if (!key.startsWith(keyPrefix))
+            continue;
+        bool ok;
+        int count = key.mid(keyPrefix.count()).toInt(&ok);
+        if (!ok || count < 0)
+            continue;
 
         const QVariantMap qtversionMap = data.value(key).toMap();
-        const QString type = qtversionMap.value(QTVERSION_TYPE_KEY).toString();
+        const QString type = qtversionMap.value(QLatin1String(QTVERSION_TYPE_KEY)).toString();
 
         bool restored = false;
         foreach (QtVersionFactory *f, factories) {
@@ -193,109 +229,88 @@ bool QtVersionManager::restoreQtVersions()
         if (!restored)
             qWarning("Warning: Unable to restore Qt version '%s' stored in %s.",
                      qPrintable(type),
-                     qPrintable(QDir::toNativeSeparators(settingsFileName())));
+                     qPrintable(filename.toUserOutput()));
     }
     ++m_idcount;
     return true;
 }
 
-void QtVersionManager::updateFromInstaller()
+void QtVersionManager::updateFromInstaller(bool emitSignal)
 {
-    ExtensionSystem::PluginManager *pm = ExtensionSystem::PluginManager::instance();
-    QList<QtVersionFactory *> factories = pm->getObjects<QtVersionFactory>();
-    Utils::PersistentSettingsReader reader;
-    if (!reader.load(Core::ICore::instance()->resourcePath()
-                     + QLatin1String("/Nokia") + QLatin1String(QTVERSION_FILENAME)))
-        return;
+    m_fileWatcherTimer->stop();
 
-    QVariantMap data = reader.restoreValues();
+    const Utils::FileName path = globalSettingsFileName();
+    // Handle overwritting of data:
+    if (m_configFileWatcher) {
+        m_configFileWatcher->removeFile(path.toString());
+        m_configFileWatcher->addFile(path.toString(), Utils::FileSystemWatcher::WatchModifiedDate);
+    }
+
+    QList<int> added;
+    QList<int> removed;
+    QList<int> changed;
+
+    QList<QtVersionFactory *> factories = ExtensionSystem::PluginManager::getObjects<QtVersionFactory>();
+    Utils::PersistentSettingsReader reader;
+    QVariantMap data;
+    if (reader.load(path))
+        data = reader.restoreValues();
 
     if (debug) {
         qDebug()<< "======= Existing Qt versions =======";
         foreach (BaseQtVersion *version, m_versions) {
-            qDebug() << version->qmakeCommand() << "id:"<<version->uniqueId();
+            qDebug() << version->qmakeCommand().toString() << "id:"<<version->uniqueId();
             qDebug() << "  autodetection source:"<< version->autodetectionSource();
             qDebug() << "";
         }
-    }
-
-    int oldcount = data.value(QLatin1String(OLDQTVERSION_COUNT_KEY), 0).toInt();
-    for (int i=0; i < oldcount; ++i) {
-        const QString key = QString::fromLatin1(OLDQTVERSION_DATA_KEY) +QString::number(i);
-        if (!data.contains(key))
-            break;
-        QVariantMap map = data.value(key).toMap();
-        QString path = map.value(OLDQTVERSION_PATH).toString();
-#ifdef Q_OS_WIN
-        path = path.toLower();
-#endif
-        QString autodetectionSource = map.value(OLDQTVERSION_SDKSOURCE).toString();
-        foreach (BaseQtVersion *v, m_versions) {
-            if (v->qmakeCommand() == path) {
-                if (v->autodetectionSource().isEmpty()) {
-                    v->setAutoDetectionSource(autodetectionSource);
-                } else {
-                    if (debug)
-                        qDebug() << "## Conflicting autodetictonSource for"<<path<<"\n"
-                                 <<"     version retains"<<v->autodetectionSource();
-                }
-                // No break, we want to mark all qt versions matching that path
-                // There's no way for us to decide whether this qt was added
-                // by the user or by the installer, so we treat them all as coming
-                // from the installer. Thus removing/updating them deletes/updates them all
-                // Note: This only applies to versions that are marked via QtVersion.Old
-            }
-        }
-    }
-
-    if (debug) {
-        qDebug()<< "======= After using OLD QtVersion data to mark versions =======";
-        foreach (BaseQtVersion *version, m_versions) {
-            qDebug() << version->qmakeCommand() << "id:"<<version->uniqueId();
-            qDebug() << "  autodetection source:"<< version->autodetectionSource();
-            qDebug() << "";
-        }
-
         qDebug()<< "======= Adding sdk versions =======";
     }
+
     QStringList sdkVersions;
-    int count = data.value(QLatin1String(QTVERSION_COUNT_KEY), 0).toInt();
-    for (int i = 0; i < count; ++i) {
-        const QString key = QString::fromLatin1(QTVERSION_DATA_KEY) + QString::number(i);
-        if (!data.contains(key))
-            break;
+
+    const QString keyPrefix = QLatin1String(QTVERSION_DATA_KEY);
+    foreach (const QString &key, data.keys()) {
+        if (!key.startsWith(keyPrefix))
+            continue;
+        bool ok;
+        int count = key.mid(keyPrefix.count()).toInt(&ok);
+        if (!ok || count < 0)
+            continue;
 
         QVariantMap qtversionMap = data.value(key).toMap();
-        const QString type = qtversionMap.value(QTVERSION_TYPE_KEY).toString();
+        const QString type = qtversionMap.value(QLatin1String(QTVERSION_TYPE_KEY)).toString();
         const QString autoDetectionSource = qtversionMap.value(QLatin1String("autodetectionSource")).toString();
         sdkVersions << autoDetectionSource;
         int id = -1; // see BaseQtVersion::fromMap()
         QtVersionFactory *factory = 0;
         foreach (QtVersionFactory *f, factories) {
-            if (f->canRestore(type)) {
+            if (f->canRestore(type))
                 factory = f;
-            }
         }
         if (!factory) {
             if (debug)
                 qDebug("Warning: Unable to find factory for type '%s'", qPrintable(type));
             continue;
         }
-        // First try to find a existing qt version to update
+        // First try to find a existing Qt version to update
         bool restored = false;
         foreach (BaseQtVersion *v, m_versions) {
             if (v->autodetectionSource() == autoDetectionSource) {
                 id = v->uniqueId();
                 if (debug)
                     qDebug() << " Qt version found with same autodetection source" << autoDetectionSource << " => Migrating id:" << id;
-                removeVersion(v);
+                m_versions.remove(id);
                 qtversionMap[QLatin1String("Id")] = id;
 
                 if (BaseQtVersion *qtv = factory->restore(type, qtversionMap)) {
                     Q_ASSERT(qtv->isAutodetected());
-                    addVersion(qtv);
+                    m_versions.insert(id, qtv);
                     restored = true;
                 }
+                if (restored)
+                    changed << id;
+                else
+                    removed << id;
             }
         }
         // Create a new qtversion
@@ -304,7 +319,8 @@ void QtVersionManager::updateFromInstaller()
                 qDebug() << " No Qt version found matching" << autoDetectionSource << " => Creating new version";
             if (BaseQtVersion *qtv = factory->restore(type, qtversionMap)) {
                 Q_ASSERT(qtv->isAutodetected());
-                addVersion(qtv);
+                m_versions.insert(qtv->uniqueId(), qtv);
+                added << qtv->uniqueId();
                 restored = true;
             }
         }
@@ -317,17 +333,18 @@ void QtVersionManager::updateFromInstaller()
     if (debug) {
         qDebug() << "======= Before removing outdated sdk versions =======";
         foreach (BaseQtVersion *version, m_versions) {
-            qDebug() << version->qmakeCommand() << "id:"<<version->uniqueId();
+            qDebug() << version->qmakeCommand().toString() << "id:"<<version->uniqueId();
             qDebug() << "  autodetection source:"<< version->autodetectionSource();
             qDebug() << "";
         }
     }
     foreach (BaseQtVersion *qtVersion, QtVersionManager::instance()->versions()) {
-        if (qtVersion->autodetectionSource().startsWith("SDK.")) {
+        if (qtVersion->autodetectionSource().startsWith(QLatin1String("SDK."))) {
             if (!sdkVersions.contains(qtVersion->autodetectionSource())) {
                 if (debug)
                     qDebug() << "  removing version"<<qtVersion->autodetectionSource();
-                removeVersion(qtVersion);
+                m_versions.remove(qtVersion->uniqueId());
+                removed << qtVersion->uniqueId();
             }
         }
     }
@@ -335,101 +352,48 @@ void QtVersionManager::updateFromInstaller()
     if (debug) {
         qDebug()<< "======= End result =======";
         foreach (BaseQtVersion *version, m_versions) {
-            qDebug() << version->qmakeCommand() << "id:"<<version->uniqueId();
+            qDebug() << version->qmakeCommand().toString() << "id:"<<version->uniqueId();
             qDebug() << "  autodetection source:"<< version->autodetectionSource();
             qDebug() << "";
         }
     }
+    if (emitSignal)
+        emit qtVersionsChanged(added, removed, changed);
+    saveQtVersions();
 }
 
 void QtVersionManager::saveQtVersions()
 {
-    Utils::PersistentSettingsWriter writer;
-    writer.saveValue(QLatin1String(QTVERSION_FILE_VERSION_KEY), 1);
+    if (!m_writer)
+        return;
+
+    QVariantMap data;
+    data.insert(QLatin1String(QTVERSION_FILE_VERSION_KEY), 1);
 
     int count = 0;
     foreach (BaseQtVersion *qtv, m_versions) {
         QVariantMap tmp = qtv->toMap();
         if (tmp.isEmpty())
             continue;
-        tmp.insert(QTVERSION_TYPE_KEY, qtv->type());
-        writer.saveValue(QString::fromLatin1(QTVERSION_DATA_KEY) + QString::number(count), tmp);
+        tmp.insert(QLatin1String(QTVERSION_TYPE_KEY), qtv->type());
+        data.insert(QString::fromLatin1(QTVERSION_DATA_KEY) + QString::number(count), tmp);
         ++count;
 
     }
-    writer.saveValue(QLatin1String(QTVERSION_COUNT_KEY), count);
-    writer.save(settingsFileName(), "QtCreatorQtVersions", Core::ICore::instance()->mainWindow());
+    m_writer->save(data, Core::ICore::mainWindow());
 }
 
 void QtVersionManager::findSystemQt()
 {
-    QString systemQMakePath = ProjectExplorer::DebuggingHelperLibrary::findSystemQt(Utils::Environment::systemEnvironment());
+    Utils::FileName systemQMakePath = QtSupport::DebuggingHelperLibrary::findSystemQt(Utils::Environment::systemEnvironment());
     if (systemQMakePath.isNull())
         return;
 
     BaseQtVersion *version = QtVersionFactory::createQtVersionFromQMakePath(systemQMakePath);
-    version->setDisplayName(BaseQtVersion::defaultDisplayName(version->qtVersionString(), systemQMakePath, true));
-    m_versions.insert(version->uniqueId(), version);
-}
-
-bool QtVersionManager::legacyRestore()
-{
-    QSettings *s = Core::ICore::instance()->settings();
-    if (!s->contains(QLatin1String(QtVersionsSectionName) + QLatin1String("/size")))
-        return false;
-    int size = s->beginReadArray(QtVersionsSectionName);
-    for (int i = 0; i < size; ++i) {
-        s->setArrayIndex(i);
-        // Find the right id
-        // Either something saved or something generated
-        // Note: This code assumes that either all ids are read from the settings
-        // or generated on the fly.
-        int id = s->value("Id", -1).toInt();
-        if (id == -1)
-            id = getUniqueId();
-        else if (m_idcount < id)
-            m_idcount = id + 1;
-
-        QString qmakePath = s->value("QMakePath").toString();
-        if (qmakePath.isEmpty())
-            continue; //skip this version
-
-        BaseQtVersion *version = QtVersionFactory::createQtVersionFromLegacySettings(qmakePath, id, s);
-        if (!version) // Likely to be a invalid version
-            continue;
-
-        if (m_versions.contains(version->uniqueId())) {
-            // oh uh;
-            delete version;
-        } else {
-            m_versions.insert(version->uniqueId(), version);
-        }
-        // Update from 2.1 or earlier:
-        QString mingwDir = s->value(QLatin1String("MingwDirectory")).toString();
-        if (!mingwDir.isEmpty()) {
-            QFileInfo fi(mingwDir + QLatin1String("/bin/g++.exe"));
-            if (fi.exists() && fi.isExecutable()) {
-                ProjectExplorer::MingwToolChain *tc = createToolChain<ProjectExplorer::MingwToolChain>(ProjectExplorer::Constants::MINGW_TOOLCHAIN_ID);
-                if (tc) {
-                    tc->setCompilerPath(fi.absoluteFilePath());
-                    tc->setDisplayName(tr("MinGW from %1").arg(version->displayName()));
-                    // The debugger is set later in the autoDetect method of the MinGw tool chain factory
-                    // as the default debuggers are not yet registered.
-                    ProjectExplorer::ToolChainManager::instance()->registerToolChain(tc);
-                }
-            }
-        }
-        const QString mwcDir = s->value(QLatin1String("MwcDirectory")).toString();
-        if (!mwcDir.isEmpty())
-            m_pendingMwcUpdates.append(mwcDir);
-        const QString gcceDir = s->value(QLatin1String("GcceDirectory")).toString();
-        if (!gcceDir.isEmpty())
-            m_pendingGcceUpdates.append(gcceDir);
-
+    if (version) {
+        version->setDisplayName(BaseQtVersion::defaultDisplayName(version->qtVersionString(), systemQMakePath, true));
+        m_versions.insert(version->uniqueId(), version);
     }
-    s->endArray();
-    s->remove(QtVersionsSectionName);
-    return true;
 }
 
 void QtVersionManager::addVersion(BaseQtVersion *version)
@@ -441,7 +405,7 @@ void QtVersionManager::addVersion(BaseQtVersion *version)
     int uniqueId = version->uniqueId();
     m_versions.insert(uniqueId, version);
 
-    emit qtVersionsChanged(QList<int>() << uniqueId);
+    emit qtVersionsChanged(QList<int>() << uniqueId, QList<int>(), QList<int>());
     saveQtVersions();
 }
 
@@ -449,56 +413,30 @@ void QtVersionManager::removeVersion(BaseQtVersion *version)
 {
     QTC_ASSERT(version != 0, return);
     m_versions.remove(version->uniqueId());
-    emit qtVersionsChanged(QList<int>() << version->uniqueId());
+    emit qtVersionsChanged(QList<int>(), QList<int>() << version->uniqueId(), QList<int>());
     saveQtVersions();
     delete version;
 }
 
-bool QtVersionManager::supportsTargetId(const QString &id) const
-{
-    QList<BaseQtVersion *> versions = QtVersionManager::instance()->versionsForTargetId(id);
-    foreach (BaseQtVersion *v, versions)
-        if (v->isValid() && v->toolChainAvailable(id))
-            return true;
-    return false;
-}
-
-QList<BaseQtVersion *> QtVersionManager::versionsForTargetId(const QString &id, const QtVersionNumber &minimumQtVersion) const
-{
-    QList<BaseQtVersion *> targetVersions;
-    foreach (BaseQtVersion *version, m_versions) {
-        if (version->supportsTargetId(id) && version->qtVersion() >= minimumQtVersion)
-            targetVersions.append(version);
-    }
-    qSort(targetVersions.begin(), targetVersions.end(), &qtVersionNumberCompare);
-    return targetVersions;
-}
-
-QSet<QString> QtVersionManager::supportedTargetIds() const
-{
-    QSet<QString> results;
-    foreach (BaseQtVersion *version, m_versions)
-        results.unite(version->supportedTargetIds());
-    return results;
-}
-
 void QtVersionManager::updateDocumentation()
 {
     Core::HelpManager *helpManager = Core::HelpManager::instance();
     Q_ASSERT(helpManager);
     QStringList files;
     foreach (BaseQtVersion *v, m_versions) {
-        const QString docPath = v->documentationPath() + QLatin1String("/qch/");
-        const QDir versionHelpDir(docPath);
-        foreach (const QString &helpFile,
-                versionHelpDir.entryList(QStringList() << QLatin1String("*.qch"), QDir::Files))
-            files << docPath + helpFile;
-
+        const QStringList docPaths = QStringList() << v->documentationPath() + QLatin1Char('/')
+                                                   << v->documentationPath() + QLatin1String("/qch/");
+        foreach (const QString &docPath, docPaths) {
+            const QDir versionHelpDir(docPath);
+            foreach (const QString &helpFile,
+                     versionHelpDir.entryList(QStringList() << QLatin1String("*.qch"), QDir::Files))
+                files << docPath + helpFile;
+        }
     }
     helpManager->registerDocumentation(files);
 }
 
-void QtVersionManager::updateDumpFor(const QString &qmakeCommand)
+void QtVersionManager::updateDumpFor(const Utils::FileName &qmakeCommand)
 {
     foreach (BaseQtVersion *v, versions()) {
         if (v->qmakeCommand() == qmakeCommand)
@@ -507,54 +445,6 @@ void QtVersionManager::updateDumpFor(const QString &qmakeCommand)
     emit dumpUpdatedFor(qmakeCommand);
 }
 
-void QtVersionManager::updateSettings()
-{
-    updateDocumentation();
-
-    BaseQtVersion *version = 0;
-    QList<BaseQtVersion *> candidates;
-
-    // try to find a version which has both, demos and examples
-    foreach (BaseQtVersion *version, m_versions) {
-        if (version && version->hasExamples() && version->hasDemos())
-            candidates.append(version);
-    }
-
-    // in SDKs, we want to prefer the Qt version shipping with the SDK
-    QSettings *settings = Core::ICore::instance()->settings();
-    QString preferred = settings->value(QLatin1String("PreferredQMakePath")).toString();
-    preferred = QDir::fromNativeSeparators(preferred);
-    if (!preferred.isEmpty()) {
-#ifdef Q_OS_WIN
-        preferred = preferred.toLower();
-        if (!preferred.endsWith(QLatin1String(".exe")))
-            preferred.append(QLatin1String(".exe"));
-#endif
-        foreach (version, candidates) {
-            if (version->qmakeCommand() == preferred) {
-                emit updateExamples(version->examplesPath(), version->demosPath(), version->sourcePath());
-                return;
-            }
-        }
-    }
-
-    // prefer versions with declarative examples
-    foreach (version, candidates) {
-        if (QDir(version->examplesPath()+"/declarative").exists()) {
-            emit updateExamples(version->examplesPath(), version->demosPath(), version->sourcePath());
-            return;
-        }
-    }
-
-    if (!candidates.isEmpty()) {
-        version = candidates.first();
-        emit updateExamples(version->examplesPath(), version->demosPath(), version->sourcePath());
-        return;
-    }
-    return;
-
-}
-
 int QtVersionManager::getUniqueId()
 {
     return m_idcount++;
@@ -599,6 +489,37 @@ QString QtVersionManager::popPendingGcceUpdate()
     return m_pendingGcceUpdates.takeFirst();
 }
 
+Core::FeatureSet QtVersionManager::availableFeatures(const QString &platformName) const
+{
+    Core::FeatureSet features;
+    foreach (BaseQtVersion *const qtVersion, validVersions()) {
+        if (qtVersion->isValid() && ((qtVersion->platformName() == platformName) || platformName.isEmpty()))
+            features |= qtVersion->availableFeatures();
+    }
+
+    return features;
+}
+
+QStringList QtVersionManager::availablePlatforms() const
+{
+    QStringList platforms;
+    foreach (BaseQtVersion *const qtVersion, validVersions()) {
+        if (qtVersion->isValid() && !qtVersion->platformName().isEmpty())
+            platforms.append(qtVersion->platformName());
+    }
+    platforms.removeDuplicates();
+    return platforms;
+}
+
+QString QtVersionManager::displayNameForPlatform(const QString &string) const
+{
+    foreach (BaseQtVersion *const qtVersion, validVersions()) {
+        if (qtVersion->platformName() == string)
+            return qtVersion->platformDisplayName();
+    }
+    return QString();
+}
+
 BaseQtVersion *QtVersionManager::version(int id) const
 {
     QMap<int, BaseQtVersion *>::const_iterator it = m_versions.find(id);
@@ -629,6 +550,8 @@ void QtVersionManager::setNewQtVersions(QList<BaseQtVersion *> newVersions)
     SortByUniqueId sortByUniqueId;
     qSort(sortedNewVersions.begin(), sortedNewVersions.end(), sortByUniqueId);
 
+    QList<int> addedVersions;
+    QList<int> removedVersions;
     QList<int> changedVersions;
     // So we trying to find the minimal set of changed versions,
     // iterate over both sorted list
@@ -645,10 +568,10 @@ void QtVersionManager::setNewQtVersions(QList<BaseQtVersion *> newVersions)
         int nid = (*nit)->uniqueId();
         int oid = (*oit)->uniqueId();
         if (nid < oid) {
-            changedVersions.push_back(nid);
+            addedVersions.push_back(nid);
             ++nit;
         } else if (oid < nid) {
-            changedVersions.push_back(oid);
+            removedVersions.push_back(oid);
             ++oit;
         } else {
             if (!equals(*oit, *nit))
@@ -659,12 +582,12 @@ void QtVersionManager::setNewQtVersions(QList<BaseQtVersion *> newVersions)
     }
 
     while (nit != nend) {
-        changedVersions.push_back((*nit)->uniqueId());
+        addedVersions.push_back((*nit)->uniqueId());
         ++nit;
     }
 
     while (oit != oend) {
-        changedVersions.push_back((*oit)->uniqueId());
+        removedVersions.push_back((*oit)->uniqueId());
         ++oit;
     }
 
@@ -673,27 +596,26 @@ void QtVersionManager::setNewQtVersions(QList<BaseQtVersion *> newVersions)
     foreach (BaseQtVersion *v, sortedNewVersions)
         m_versions.insert(v->uniqueId(), v);
 
-    if (!changedVersions.isEmpty())
+    if (!changedVersions.isEmpty() || !addedVersions.isEmpty() || !removedVersions.isEmpty())
         updateDocumentation();
 
-    updateSettings();
     saveQtVersions();
 
-    if (!changedVersions.isEmpty())
-        emit qtVersionsChanged(changedVersions);
+    if (!changedVersions.isEmpty() || !addedVersions.isEmpty() || !removedVersions.isEmpty())
+        emit qtVersionsChanged(addedVersions, removedVersions, changedVersions);
 }
 
 // Returns the version that was used to build the project in that directory
 // That is returns the directory
 // To find out whether we already have a qtversion for that directory call
 // QtVersion *QtVersionManager::qtVersionForDirectory(const QString directory);
-QString QtVersionManager::findQMakeBinaryFromMakefile(const QString &makefile)
+Utils::FileName QtVersionManager::findQMakeBinaryFromMakefile(const QString &makefile)
 {
     bool debugAdding = false;
     QFile fi(makefile);
     if (fi.exists() && fi.open(QFile::ReadOnly)) {
         QTextStream ts(&fi);
-        QRegExp r1("QMAKE\\s*=(.*)");
+        QRegExp r1(QLatin1String("QMAKE\\s*=(.*)"));
         while (!ts.atEnd()) {
             QString line = ts.readLine();
             if (r1.exactMatch(line)) {
@@ -707,28 +629,23 @@ QString QtVersionManager::findQMakeBinaryFromMakefile(const QString &makefile)
 #endif
                 // Is qmake still installed?
                 QFileInfo fi(qmakePath);
-                if (fi.exists()) {
-                    qmakePath = fi.absoluteFilePath();
-#ifdef Q_OS_WIN
-                    qmakePath = qmakePath.toLower();
-#endif
-                    return qmakePath;
-                }
+                if (fi.exists())
+                    return Utils::FileName(fi);
             }
         }
     }
-    return QString();
+    return Utils::FileName();
 }
 
-BaseQtVersion *QtVersionManager::qtVersionForQMakeBinary(const QString &qmakePath)
+BaseQtVersion *QtVersionManager::qtVersionForQMakeBinary(const Utils::FileName &qmakePath)
 {
-   foreach (BaseQtVersion *version, versions()) {
-       if (version->qmakeCommand() == qmakePath) {
-           return version;
-           break;
-       }
-   }
-   return 0;
+    foreach (BaseQtVersion *version, versions()) {
+        if (version->qmakeCommand() == qmakePath) {
+            return version;
+            break;
+        }
+    }
+    return 0;
 }
 
 void dumpQMakeAssignments(const QList<QMakeAssignment> &list)
@@ -743,11 +660,15 @@ QtVersionManager::MakefileCompatible QtVersionManager::makefileIsFor(const QStri
     if (proFile.isEmpty())
         return CouldNotParse;
 
+    // The Makefile.Debug / Makefile.Release lack a # Command: line
+    if (findQMakeLine(makefile, QLatin1String("# Command:")).trimmed().isEmpty())
+        return CouldNotParse;
+
     QString line = findQMakeLine(makefile, QLatin1String("# Project:")).trimmed();
     if (line.isEmpty())
         return CouldNotParse;
 
-    line = line.mid(line.indexOf(QChar(':')) + 1);
+    line.remove(0, line.indexOf(QLatin1Char(':')) + 1);
     line = line.trimmed();
 
     QFileInfo srcFileInfo(QFileInfo(makefile).absoluteDir(), line);
@@ -798,7 +719,7 @@ QPair<BaseQtVersion::QmakeBuildConfigs, QString> QtVersionManager::scanMakeFile(
     if (debug) {
         qDebug()<<"\n\nDumping information from scanMakeFile";
         qDebug()<<"QMake CONFIG variable parsing";
-        qDebug()<<"  "<< (result & BaseQtVersion::NoBuild ? "No Build" : QString::number(int(result)));
+        qDebug()<<"  "<< (result & BaseQtVersion::NoBuild ? QByteArray("No Build") : QByteArray::number(int(result)));
         qDebug()<<"  "<< (result & BaseQtVersion::DebugBuild ? "debug" : "release");
         qDebug()<<"  "<< (result & BaseQtVersion::BuildAll ? "debug_and_release" : "no debug_and_release");
         qDebug()<<"\nAddtional Arguments";
@@ -833,7 +754,7 @@ QString QtVersionManager::trimLine(const QString line)
 
 void QtVersionManager::parseArgs(const QString &args, QList<QMakeAssignment> *assignments, QList<QMakeAssignment> *afterAssignments, QString *additionalArguments)
 {
-    QRegExp regExp("([^\\s\\+-]*)\\s*(\\+=|=|-=|~=)(.*)");
+    QRegExp regExp(QLatin1String("([^\\s\\+-]*)\\s*(\\+=|=|-=|~=)(.*)"));
     bool after = false;
     bool ignoreNext = false;
     *additionalArguments = args;
@@ -885,22 +806,22 @@ BaseQtVersion::QmakeBuildConfigs QtVersionManager::qmakeBuildConfigFromCmdArgs(Q
     QList<QMakeAssignment> oldAssignments = *assignments;
     assignments->clear();
     foreach(const QMakeAssignment &qa, oldAssignments) {
-        if (qa.variable == "CONFIG") {
-            QStringList values = qa.value.split(' ');
+        if (qa.variable == QLatin1String("CONFIG")) {
+            QStringList values = qa.value.split(QLatin1Char(' '));
             QStringList newValues;
             foreach(const QString &value, values) {
-                if (value == "debug") {
-                    if (qa.op == "+=")
+                if (value == QLatin1String("debug")) {
+                    if (qa.op == QLatin1String("+="))
                         result = result  | BaseQtVersion::DebugBuild;
                     else
                         result = result  & ~BaseQtVersion::DebugBuild;
-                } else if (value == "release") {
-                    if (qa.op == "+=")
+                } else if (value == QLatin1String("release")) {
+                    if (qa.op == QLatin1String("+="))
                         result = result & ~BaseQtVersion::DebugBuild;
                     else
                         result = result | BaseQtVersion::DebugBuild;
-                } else if (value == "debug_and_release") {
-                    if (qa.op == "+=")
+                } else if (value == QLatin1String("debug_and_release")) {
+                    if (qa.op == QLatin1String("+="))
                         result = result | BaseQtVersion::BuildAll;
                     else
                         result = result & ~BaseQtVersion::BuildAll;
@@ -908,7 +829,7 @@ BaseQtVersion::QmakeBuildConfigs QtVersionManager::qmakeBuildConfigFromCmdArgs(Q
                     newValues.append(value);
                 }
                 QMakeAssignment newQA = qa;
-                newQA.value = newValues.join(" ");
+                newQA.value = newValues.join(QLatin1String(" "));
                 if (!newValues.isEmpty())
                     assignments->append(newQA);
             }
@@ -918,3 +839,18 @@ BaseQtVersion::QmakeBuildConfigs QtVersionManager::qmakeBuildConfigFromCmdArgs(Q
     }
     return result;
 }
+
+Core::FeatureSet QtFeatureProvider::availableFeatures(const QString &platformName) const
+{
+     return QtVersionManager::instance()->availableFeatures(platformName);
+}
+
+QStringList QtFeatureProvider::availablePlatforms() const
+{
+    return QtVersionManager::instance()->availablePlatforms();
+}
+
+QString QtFeatureProvider::displayNameForPlatform(const QString &string) const
+{
+    return QtVersionManager::instance()->displayNameForPlatform(string);
+}
